/* ============================================================================================================================================= /
 * PROJECT:         Modulo remoto per stazione meteo 
 * FILE:            Main.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F26K22
 * CREATED:         08/12/2016        
 * UPDATED:         28/02/2016
 *
 * DESCRIP:         
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 /* ============================================================================================================================================ *

 /* I/O PIC18F26K22============================================================================================================================= /
 * RA0      GPIO 0 
 * RA1      GPIO 1
 * RA2      GPIO 2
 * RA3      GPIO 3
 * RA4      GPIO 4
 * RA5      V_BATT   
 * RB0      LED_1
 * RB1      SCK SPI 2 
 * RB2      SDI SPI 2
 * RB3      SDO SPI 2
 * RB4:     NC
 * RB5:     NC
 * RB6:     PGCK
 * RB7:     PGDAT
 * RC0:     HM_TR P_EN   
 * RC1:     HM_TRP_CONF
 * RC2:     NC
 * RC3:     SCL I2C_1
 * RC4:     SDA I2C_1
 * RC6:     TX UART_1
 * RC7:     RX UART_1
 */

/* CONFIGURATION BITS ========================================================================================================================= */

// CONFIG1H
#pragma config FOSC = INTIO67   // Oscillator Selection bits (Internal oscillator block)
//#pragma config FOSC = INTIO7
#pragma config PLLCFG = OFF     // 4X PLL Disabled
#pragma config PRICLKEN = ON    // Primary clock enable bit (Primary clock enabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRTEN = OFF     // Power-up Timer Enable bit (Power up timer disabled)
#pragma config BOREN = SBORDIS  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
#pragma config BORV = 190       // Brown Out Reset Voltage bits (VBOR set to 1.90 V nominal)

// CONFIG2H
#pragma config WDTEN = OFF       // Watchdog Timer Enable bits (WDT is always enabled. SWDTEN bit has no effect)
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = PORTC1  // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF     // PORTB A/D Enable bit (PORTB<5:0> pins are configured as analog input channels on Reset)
#pragma config CCP3MX = PORTB5  // P3A/CCP3 Mux bit (P3A/CCP3 input/output is multiplexed with RB5)
#pragma config HFOFST = ON      // HFINTOSC Fast Start-up (HFINTOSC output and ready status are not delayed by the oscillator stable status)
#pragma config T3CMX = PORTC0   // Timer3 Clock input mux bit (T3CKI is on RC0)
#pragma config P2BMX = PORTB5   // ECCP2 B output mux bit (P2B is on RB5)
#pragma config MCLRE = EXTMCLR  // MCLR Pin Enable bit (MCLR pin enabled, RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = ON         // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection Block 0 (Block 0 (000800-003FFFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection Block 1 (Block 1 (004000-007FFFh) not code-protected)
#pragma config CP2 = OFF        // Code Protection Block 2 (Block 2 (008000-00BFFFh) not code-protected)
#pragma config CP3 = OFF        // Code Protection Block 3 (Block 3 (00C000-00FFFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection Block 0 (Block 0 (000800-003FFFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection Block 1 (Block 1 (004000-007FFFh) not write-protected)
#pragma config WRT2 = OFF       // Write Protection Block 2 (Block 2 (008000-00BFFFh) not write-protected)
#pragma config WRT3 = OFF       // Write Protection Block 3 (Block 3 (00C000-00FFFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0007FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection Block 0 (Block 0 (000800-003FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection Block 1 (Block 1 (004000-007FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection Block 2 (Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection Block 3 (Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0007FFh) not protected from table reads executed in other blocks)

/* COSTANTI ===================================================================================================================================== */
#define XTAL_FREQ           1                                                           // definisco la frequenza per Libreria Delay

#define ON                  1
#define OFF                 0

#define LED_1               LATBbits.LATB0

// tempo durate sleep, invio dati ogni
#define TMR0_PRELOAD_8  8346  // 30 MINUTI  
#define TMR0_PRELOAD_7  27511 // 20 MINUTI
#define TMR0_PRELOAD_6  46544 // 10 MINUTI 
#define TMR0_PRELOAD_5  56070 // 5 MINUTI
#define TMR0_PRELOAD_4  63674 // 1 MINUTO
#define TMR0_PRELOAD_3  64626 // 30 SECONDI
#define TMR0_PRELOAD_2  65270 // 10 SECONDI
#define TMR0_PRELOAD_1  65420 // 5 SECONDI

/* PERSONALIZZAZIONI ============================================================================================================================= */
#define VBAT_L1             372     //1.2V    =(1.2/3.3)*1024 
#define VBAT_L2             553     //1.8V
#define VBAT_L3             744     //2.4V
#define VBAT_L4             1000    //3.2V

#define LPS25HB_ONE_POINT_CAL  -8.09  //-9.24

#define TIMEOUT_SETTINGS    3
/* ============================================================================================================================================= /
 * FUNCTION:        DefineIO
 * Overview:        Definizione e inizializzazione ingressi e uscite
/* ============================================================================================================================================ */

void Define_IO(void) {
    LATA =      0b00000000;                                                             // Reset Latch uscite
    ANSELA =    0b00000000;                                                             // RA5  impostato come uscita, viene definito con ingresso analogico 
    TRISA =     0b00000000;                                                             // nella funzione Read_vbat
    LATB =      0b00000000;                                                             // Reset Latch uscite
    ANSELB =    0b00000000;
    TRISB =     0b00000100;
    LATC =      0b00000000;                                                             // Reset Latch uscite
    ANSELC =    0b00000000;
    TRISC =     0b11011000;
}

/* ============================================================================================================================================= /
 * FUNCTION:        Module_settings
 * Overview:        Configurazioni dei moduli interni
/* ============================================================================================================================================ */

void Module_settings(void) {
    INTCON2bits.RBPU = 1;                                                               // pullup disabilitate
    
    OSCCONbits.IRCF0 = 1;                                                               // oscillatore interno 1 MHz
    OSCCONbits.IRCF1 = 1;    
    OSCCONbits.IRCF2 = 0;    
    OSCTUNEbits.PLLEN = 0;                                                              //0= PLL disabilitata

    CCP2CONbits.CCP2M0 = 0;
    CCP2CONbits.CCP2M1 = 0;
    CCP2CONbits.CCP2M2 = 0;
    CCP2CONbits.CCP2M3 = 0;
    
    CCP1CONbits.CCP1M0 = 0;                                                             // disabilito tutti i comparatori
    CCP1CONbits.CCP1M1 = 0;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M3 = 0;

    CCP3CONbits.CCP3M0 = 0;
    CCP3CONbits.CCP3M1 = 0;
    CCP3CONbits.CCP3M2 = 0;
    CCP3CONbits.CCP3M3 = 0;
    
    CCP4CONbits.CCP4M0 = 0;
    CCP4CONbits.CCP4M1 = 0;
    CCP4CONbits.CCP4M2 = 0;
    CCP4CONbits.CCP4M3 = 0;
        
    CCP5CONbits.CCP5M0 = 0;
    CCP5CONbits.CCP5M1 = 0;
    CCP5CONbits.CCP5M2 = 0;
    CCP5CONbits.CCP5M3 = 0;
    
    // POWER MANAGEMENT
    OSCCONbits.IDLEN = 1;                                                               // modalit IDLE (periferiche attive)
    
    PMD0bits.UART2MD = 1;                                                               // disabilito i moduli non utilizzati
    PMD0bits.TMR2MD = 1;
    PMD0bits.TMR3MD = 1;
    PMD0bits.TMR4MD = 1;
    PMD0bits.TMR5MD = 1;
    PMD0bits.TMR6MD = 1;
    
    PMD1bits.CCP1MD = 1;
    PMD1bits.CCP2MD = 1;
    PMD1bits.CCP3MD = 1;
    PMD1bits.CCP4MD = 1;
    PMD1bits.CCP5MD = 1;
    PMD1bits.MSSP2MD = 1;

    PMD2bits.CMP1MD = 1;
    PMD2bits.CMP2MD = 1;
    PMD2bits.CTMUMD = 1;
    
/* INTERRUPT GLOBALE e PRIORITA'*/
    RCONbits.IPEN = 1;
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
    
/* Impostazioni ingresso analogico */    
    ADCON0bits.CHS = 4;     // AN4 selected
    ADCON1bits.PVCFG = 0;   // +VREF = VDD (3.3V)
    ADCON1bits.NVCFG = 0;   // -VREF = GND
    ADCON2bits.ACQT0 = 0;   // 8 TAD
    ADCON2bits.ACQT1 = 0;
    ADCON2bits.ACQT2 = 1;
    ADCON2bits.ADCS0 = 0;   // fosc/2
    ADCON2bits.ADCS1 = 0;
    ADCON2bits.ADCS2 = 0;
    ADCON2bits.ADFM = 1;    // right justified
//    ADCON0bits.ADON = 1;

// TIMER 0, Configurazioni per avere 1 interrupt ogni 10 minuti = (65536-preload)*(4/fosc)*prescaler = (65536-47225)*(4/31250)*256
    T0CONbits.T0PS0 = 1;                                                                // prescaler 1/256
    T0CONbits.T0PS1 = 1;                                                                //
    T0CONbits.T0PS2 = 1;
    T0CONbits.PSA = 0;                                                                  // prescaler assegnato
    T0CONbits.T0CS = 0;                                                                 // da clock interno fosc/4
    T0CONbits.T08BIT = 0;                                                               // 16 BIT
//    TMR0 = TMR0_PRELOAD;                                                                // preload timer 0
    
    INTCONbits.TMR0IE = 1;
    INTCONbits.TMR0IF = 0;
    INTCON2bits.TMR0IP = 1;

// TIMER 1, Configurazioni per avere 1 interrupt ogni 1 secondo = (65536-preload)*(4/fosc)*prescaler = (65536-34286)*(4/1000000)*8
    
    T1CONbits.TMR1CS = 0;                                                               // da clock interno fosc/4
    T1CONbits.T1CKPS0 = 1;                                                              // prescaler 1/8
    T1CONbits.T1CKPS1 = 1;                                                              
    T1CONbits.T1RD16 = 1;                                                               // 16 bit
    
    TMR1 = 34286;                                                                       // preload timer 0
    
    T1CONbits.TMR1ON = 0;
    
    PIR1bits.TMR1IF = 0;
    IPR1bits.TMR1IP = 0;
    PIE1bits.TMR1IE = 1;
}