/* ============================================================================================================================================= /
 * LIBRARY:         HMTRP
 * FILE:            HMTRP.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         13/02/2017
 * UPDATED:         
 *
 * DESCRIP:         Gestione modulo HMTRP 433-868 MHz
 *
 * PRECONDITIONS:  
 * FUNCTIONS:       
 *
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               15/02/2017          - Prima build solo per gestione avvio, invio e ricezione dati
  1.0               13/02/2017          - Inizio studio modulo HMTRP
//! [History]
/* ============================================================================================================================================ */
#ifndef HMTRP
#define HMTRP

/* INCLUDES =================================================================================================================================== */
#include "HMTRP_settings.h"
#include "HMTRP.h"

/* VARIABILI GLOBALI ========================================================================================================================== */
// variabili globali perch comunque posso ricevere e verificare solo una risposta alla volta.
//  inutile usare puntatori o passare le variabili alle funzioni
volatile unsigned int hmtrp_buffer_index = 0;
unsigned char hmtrp_buffer[HMTRP_BUFFER_LENGTH] = 0;
volatile bit hmtrp_string_received = 0;

/* ============================================================================================================================================= /
 * FUNCTION:        HMTRP_init
 * 
 * Overview:        Imposta le linee di ricezione, abilita e resetta il modulo HMTRP
/* ============================================================================================================================================ */
void HMTRP_init(void){
   
    // assicurarsi che siano abilitati interrupt globali
    // abilitazione interrupt in ricezione in base al modulo UART selezionato
    HMTRP_CONFIG = 1;
    HMTRP_ENABLE = 0;
    
    Delay_ms(200);
    HMTRP_reset_buffer();
    
    HMTRP_INTERRUPT_IP = HMTRP_INT_PRIORITY;
    HMTRP_INTERRUPT_IF = 0;
    HMTRP_INTERRUPT_IE = 1;
    
// All'avvio ricevo HM-TRP-V02! ma se il modulo si accende insieme al PIC  impossibile da ricevere
// Inizializzare il modulo UART dopo questa funzione
}

/* ============================================================================================================================================= /
 * FUNCTION:        HMTRP_mode
 * Input:           _mode: modalit di funzionamento del modulo HMTRP (HMTRP_SLEEP_MODE/HMTRP_NORMAL_MODE/HMTRP_CONFIG_MODE)
 *
 * Overview:        Imposta la modalit di funzionamento del modulo HMTRP
 * Nota:            Anche se la modalit configurazione  impostabile al momento non ci sono funzioni per leggere e scrivere i parametri del modulo HMTRP
/* ============================================================================================================================================ */
void HMTRP_mode(unsigned char _mode){
    // assicurarsi che siano abilitati interrupt globali
    // abilitazione interrupt in ricezione in base al modulo UART selezionato

    switch (_mode){
        case HMTRP_SLEEP_MODE:  HMTRP_ENABLE = 1;
                                HMTRP_CONFIG = 1;
                                HMTRP_INTERRUPT_IE = 0;
                                break;
        case HMTRP_NORMAL_MODE: HMTRP_ENABLE = 0;
                                HMTRP_CONFIG = 1;
                                HMTRP_INTERRUPT_IE = 1;
                                break;  
        case HMTRP_CONFIG_MODE: HMTRP_ENABLE = 0;
                                HMTRP_CONFIG = 0;
                                HMTRP_INTERRUPT_IE = 1;
                                break;                              
        
    }
}
/* ============================================================================================================================================= /
 * FUNCTION:        HMTRP_reset_buffer
 *
 * Overview:        Azzera il buffer di ricezione
/* ============================================================================================================================================ */
void HMTRP_reset_buffer(void){
    hmtrp_buffer_index = 0;                                                             // azzero indice buffer e flag data_ready
    hmtrp_string_received = 0;

    for (unsigned int _i = 0; _i < HMTRP_BUFFER_LENGTH; _i++)                           // azzero l'array della risposta ricevuta
    {
        hmtrp_buffer[_i] = 0;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Ciclo interrupt da integrare in main
 *
 * Overview:        Alla ricezione di un byte sulla linea UART dedicata salva il byte nel buffer di ricezione hmtrp_buffer, alla ricezione di 
 *                  0x0A 0x0D in successione viene settato il flag di stringa ricevuta
 *                  
/* ============================================================================================================================================ */
//void high_priority interrupt inter_high(void) {
//    if (HMTRP_INTERRUPT_IE && HMTRP_INTERRUPT_IF) {                                     // se l'interrupt  stato causato da una ricezione dati USART
//        
//        hmtrp_buffer[hmtrp_buffer_index] = HMTRP_receive();                             // salvo i dati ricevuti nell'array usart_received
//        
//#ifdef DEBUG_RESEND_ON_UART_RAW
//    _HMTRP_deb_USART_send_char(hmtrp_buffer[hmtrp_buffer_index]);
//#endif  
//        
//    if (hmtrp_buffer_index>0){
//        if ((hmtrp_buffer[hmtrp_buffer_index]== 0x0A) && (hmtrp_buffer[hmtrp_buffer_index-1])== 0x0D){
//            hmtrp_string_received = 1;
//            hmtrp_buffer[hmtrp_buffer_index] = 0x00;                                    // tolgo caratteri di fine stringa
//            hmtrp_buffer[hmtrp_buffer_index-1] = 0x00;
//        }
//    }
//    
//        hmtrp_buffer_index++;
//        if (hmtrp_buffer_index > HMTRP_BUFFER_LENGTH)                                   // se supero la dimensione massima resetto l'indice
//            hmtrp_buffer_index = 0;    
//    }
//}


#endif