/*******************************************************************************

Author : Mauro Laurenti
Version : 1.3

Created on Date : 04/09/2006
Last update     : 03/11/2013

CopyRight 2006-2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*/


/** @file Delay.c
 * This library allows to execute long delay indipendently from the used clock. The clock can be set at run time (no compiling time). \n
 * Files:
 * - Delay.c:  Codice funzioni. 
 * - Delay.h: Parametri e prototipi.
 * @note    Use Delay_us for us delays, (renamed microchip macro __delay_us)
 */


#ifndef DELAY_H
#define DELAY_H

#ifndef XTAL_FREQ
    #error Definire XTAL_FREQ
#endif

#define _XTAL_FREQ          (XTAL_FREQ * 1000000)                                       // mia aggiunta per adattare anche delay standard
#define Delay_us            __delay_us

#ifdef __XC8
#include <xc.h>
#endif

// Remove the comment to the DELAY_INTERRUPT_BLOCKING definition if the delay
// should lock the interrupt.
// This feature make the delay function "almost" thread safe, in case they are
// used within the main function and within the ISR at the same time.

//#define DELAY_INTERRUPT_BLOCKING

/**
 * This function creates a delay in ms up to 1 second.
 *
 * @param value_ms Delay expressed in ms [min = 1, max = 1000]  
 *
 * @return void
 *
 * @note This is a blocking function.
 * @note Typical error is between 1%-2% depending on the compiler optimization.
 *       Tests have been done using 20MHz and the Free version of C18 and XC8 compilers.
 */
void Delay_ms (unsigned int value_ms);


/**
 * This function creates a delay in seconds, up to 255s.
 *
 * @param value_s Delay expressed in s [min = 1, max = 255]  
 *
 * @return void
 *
 * @note This is a blocking function.
 * @note Typical error is between 1%-2% depending on the compiler optimization.
 *       Tests have been done using 20MHz and the Free version of C18 and XC8 compilers.
 */
void Delay_s (unsigned char value_s);


#endif