/* ============================================================================================================================================ */
/* LIBRARY:         Lettura e scrittura da EEPROM interna
/* FILE:            EEPROM.c
/* LAST REV:        1.0
/* AUTHOR:          Ivo Colleoni
/* DEVICE:          PIC18F
/* CREATED:         24/09/2013
/* UPDATED:         21/07/2016
 *
 * DESCRIP:         Parametri e prototipi per libreria TCN75A.c
/* ============================================================================================================================================ */

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file EEPROM.c
 *  Libreria per la gestione della memoria EEPROM interna al PIC micro, compatibile con PIC18, anche con pi di 256Mb di EEPROM.
 * - Per un corretto funzionamento:
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Includere la libreria Delay.c prima delle libreria EEPROM.c
 * - Files:
 *  + EEPROM.c: Codice funzioni
 *  + EEPROM.h: Parametri e prototipi
 */
 
/**
 * Restituisce il valore contenuto nell'indirizzo dichiarato
 * @param   _addr: indirizzo da leggere (unsigned int)
 * @return  valore letto (unsigned char)
 */
unsigned char EEPROM_read(unsigned int _addr);

/**
 * Salva il valore nell'indirizzo dichiarato
 * @param   _data: valore da scrivere (unsigned char)
 * @param   _addr: indirizzo in cui scrivere (unsigned int)
 * @return  void
 */
void EEPROM_write(unsigned char _data, unsigned int _addr);

/**
 * Restituisce un valore intero letto da 2 indirizzi consecutivi
 * @param   _addr: indirizzo in cui scrivere (unsigned int)
 * @return  valore letto in _addr e _addr+1 (unsigned int)
 */
unsigned int EEPROM_read_int(unsigned int _addr);

/**
 * Salva il valore intero nell'indirizzo dichiarato e in quello successivo (2 byte)
 * @param   _data: valore da scrivere (unsigned int)
 * @param   _addr: indirizzo in cui scrivere (unsigned int)
 * @return  void
 */
void EEPROM_write_int(unsigned int _data, unsigned int _addr);
