/* ============================================================================================================================================= /
 * LIBRARY:         HMTRP
 * FILE:            HMTRP.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         13/02/2017
 * UPDATED:         
 *
 * DESCRIP:         Parametri e prototipi per libreria HMTRP.c
 * ============================================================================================================================================= /
 * Change History:
 *
 *
/* ============================================================================================================================================ */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#ifndef USART                     
    #error Includere USART_init.c
#endif

#ifdef HMTRP_USART_MODULE_1                                                             // Ridefinisco le chiamate alle funzioni a seconda del modulo USART usato
    #ifndef USART_MODULE_1
        #error USART_MODULE_1 non abilitato in USART_init_settings.h
    #endif
    #define HMTRP_send_string           USART_send_string
    #define HMTRP_send_char             USART_send_char
    #define HMTRP_receive               USART_receive
    #define HMTRP_INTERRUPT_IF          USART_INTERRUPT_IF
    #define HMTRP_INTERRUPT_IE          USART_INTERRUPT_IE
    #define HMTRP_INTERRUPT_IP          USART_INTERRUPT_IP
    
    
#elif defined HMTRP_USART_MODULE_2
    #ifndef USART_MODULE_2
        #error USART_MODULE_2 non abilitato in USART_init_settings.h
    #endif
    #define HMTRP_send_string           USART2_send_string
    #define HMTRP_send_char             USART2_send_char
    #define HMTRP_receive               USART2_receive
    #define HMTRP_INTERRUPT_IF          USART2_INTERRUPT_IF
    #define HMTRP_INTERRUPT_IE          USART2_INTERRUPT_IE
    #define HMTRP_INTERRUPT_IP          USART2_INTERRUPT_IP

#else
    #error Definire il modulo USART da utilizzare per HMTRP_settings.h
#endif

// impostazioni di debug per resend dei dati ricevuti su porta UART secondaria
#ifdef DEBUG_ON_UART_1
    #ifndef USART_MODULE_1
        #error USART_MODULE_1 non abilitato in USART_init_settings.h
    #endif
    #define _HMTRP_deb_USART_send_char    USART_send_char
#elif defined DEBUG_ON_UART_2  
    #ifndef USART_MODULE_2
        #error USART_MODULE_2 non abilitato in USART_init_settings.h
    #endif  
    #define _HMTRP_deb_USART_send_char    USART2_send_char
#endif

/* COSTANTI =================================================================================================================================== */
#define ON  1
#define OFF 0

#define HMTRP_SLEEP_MODE             0
#define HMTRP_NORMAL_MODE            1
#define HMTRP_CONFIG_MODE            2


//#define HMTRP_BAUDR_9600      1
//#define HMTRP_BAUDR_19200     2
//#define HMTRP_BAUDR_57600     3
//#define HMTRP_BAUDR_115200    4

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file HMTRP.c
 * Libreria per la gestione del modulo HMTRP via USART.
 * - Per un corretto funzionamento:
 *  + Includere la libreria Delay.c prima della libreria HMTRP.c
 *  + Includere la libreria USART_init.c prima della libreria HMTRP.c 
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Impostare il modulo USART da utilizzare per il dispositivo HMTRP in HMTRP_settings.h
 *  + Abilitare il modulo USART utilizzato in USART_init_settings.h
 *  + In main inizializzare il modulo con la funzione HMTRP_init() e impostare il modulo usart tramite funzione USART_eneble(BAUDRATE) (prima la funzione e poi inizializzare il modulo) 
 *  + Abilitare gli interrupt globali con priorit
 * - Files:
 *  + HMTRP.c: Codice funzioni
 *  + HMTRP.h: Parametri e prototipi
 *  + HMTRP_settings.h impostazioni specifiche per progetto
 */


/** 
 * Imposta le linee di ricezione, abilita e resetta il modulo HMTRP
 * @param   void
 * @return  void
 */ 
void HMTRP_init(void);

/** 
 * Imposta la modalit di funzionamento del modulo HMTRP
 * @param   _mode: modalit di funzionamento del modulo HMTRP (unsigned char)
 *              - HMTRP_SLEEP_MODE
 *              - HMTRP_NORMAL_MODE
 *              - HMTRP_CONFIG_MODE
 * @return  void
 * @note    Anche se la modalit configurazione  impostabile al momento non ci sono funzioni per leggere e scrivere i parametri del modulo HMTRP
 */ 
void HMTRP_mode(unsigned char _mode);

/** 
 * Azzera il buffer di ricezione
 * @param   void
 * @return  void
 */ 
void HMTRP_reset_buffer(void);

/**
 * Invia un byte via USART e attende il flag di invio completato.
 * @param   _data: byte da inviare (unsigned char)
 * @return  void
 */

void HMTRP_send_char (unsigned char _data);
/**
 * Invia una stringa via USART.
 * @param   _string[]: Stringa da inviare
 * @return  void
 */
void HMTRP_send_string (const unsigned char _string[]) ;

/**
 * Restituisce il byte ricevuto, bloccante se non si riceve nulla.
 * @param   void
 * @return  byte ricevuto (unsigned char)
 */
unsigned char HMTRP_receive (void);


