/* ============================================================================================================================================= /
 * LIBRARY:         Gestione LPS25HB
 * FILE:            LPS25HB.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         09/09/2016
 * UPDATED:         
 *
 * DESCRIP:         parametri e prototipi per libreria LPS25HB.c
 *
/* ============================================================================================================================================= /
 *
/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */

#ifndef I2C
    #error  Includere I2C_init.c
#endif

#ifdef LPS25HB_I2C_MODULE_1                                                             // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
    #ifndef I2C_MODULE_1
        #error I2C_MODULE_1 non abilitato in I2C_init_settings.h
    #endif
    #define _LPS25HB_I2C_1ByteAddr_1ByteData_write     I2C_1ByteAddr_1ByteData_write
    #define _LPS25HB_I2C_1ByteAddr_1ByteData_read      I2C_1ByteAddr_1ByteData_read

#elif defined LPS25HB_I2C_MODULE_2
    #ifndef I2C_MODULE_2
        #error I2C_MODULE_2 non abilitato in I2C_init_settings.h
    #endif
    #define _LPS25HB_I2C_1ByteAddr_1ByteData_write     I2C2_1ByteAddr_1ByteData_write
    #define _LPS25HB_I2C_1ByteAddr_1ByteData_read      I2C2_1ByteAddr_1ByteData_read

#else
    #error Definire il modulo I2C da utilizzare per LPS25HB
#endif

/* INDIRIZZI REGISTRI ========================================================================================================================= */
#define LPS25HB_ADDR                    0xB8                                            // indirizzo LPS25HB

#define LPS25HB_REF_P_XL                0x08
#define LPS25HB_REF_P_L                 0x09
#define LPS25HB_REF_P_H                 0x0A
#define LPS25HB_WHO_AM_I_REG            0x0F   
#define LPS25HB_RES_CONF_REG            0X10
#define LPS25HB_CTRL_REG1               0x20
#define LPS25HB_CTRL_REG2               0x21
#define LPS25HB_CTRL_REG3               0x22
#define LPS25HB_CTRL_REG4               0x23
#define LPS25HB_INTERRUPT_CFG           0x24
#define LPS25HB_INT_SOURCE              0x25
#define LPS25HB_STATUS_REG              0x27
#define LPS25HB_PRESS_OUT_XL_REG        0x28
#define LPS25HB_PRESS_OUT_L_REG         0x29
#define LPS25HB_PRESS_OUT_H_REG         0x2A
#define LPS25HB_TEMP_OUT_L_REG          0x2B
#define LPS25HB_TEMP_OUT_H_REG          0x2C
#define LPS25HB_FIFO_CTRL               0x2E
#define LPS25HB_FIFO_STATUS             0x2F
#define LPS25HB_THS_P_L                 0x30
#define LPS25HB_THS_P_H                 0x31
#define LPS25HB_RPDS_L                  0x39
#define LPS25HB_RPDS_H                  0x3A

/* COSTANTI ================================================================================================================================== */

//LPS25HB_RES_CONF_REG
#define LPS25HB_TEMP_RES_8              0b00000000
#define LPS25HB_TEMP_RES_16             0b00000100
#define LPS25HB_TEMP_RES_32             0b00001000
#define LPS25HB_TEMP_RES_64             0b00001100
#define LPS25HB_PRES_RES_8              0b00000000
#define LPS25HB_PRES_RES_32             0b00000001
#define LPS25HB_PRES_RES_128            0b00000010
#define LPS25HB_PRES_RES_512            0b00000011

//LPS25HB_CTRL_REG1
#define LSP25HB_POWER_DOWN              0b00000000
#define LSP25HB_ACTIVE                  0b10000000
#define LSP25HB_ODR_ONE_SHOT            0b00000000
#define LSP25HB_ODR_1HZ                 0b00010000
#define LSP25HB_ODR_7HZ                 0b00100000
#define LSP25HB_ODR_12HZ                0b00110000
#define LSP25HB_ODR_25HZ                0b01000000
#define LPS25HB_DIFF_EN                 0b00001000
#define LPS25HB_AUTO_ZERO               0b00000010

//LPS25HB_CTRL_REG2
#define LPS25HB_ONE_SHOT                0b00000001
#define LPS25HB_SWRESET                 0b00000100
#define LPS25HB_BOOT                    0b10000000
#define LPS25HB_FIFO_EN                 0b01000000
//#define LPS25HB_AUTO_ZERO               0b00000010

//LPS25HB_CTRL_REG3
#define LPS25HB_INTS_DATA               0b00000000
#define LPS25HB_INT_P_HIGH              0b00000001
#define LPS25HB_INT_P_LOW               0b00000010
#define LPS25HB_INT_P_HIGH_LOW          0b00000011
#define LPS25HB_INT_ACTIVE_HIGH         0b00000000
#define LPS25HB_INT_ACTIVE_LOW          0b10000000
#define LPS25HB_INT_PUSH_PULL           0b00000000
#define LPS25HB_INT_OPEN_DRAIN          0b01000000

//LPS25HB_CTRL_REG4
#define LPS25HB_INT_DRDY                0b00010000 // prima di scrivere nel registro spostare >> 4
#define LPS25HB_INT_F_OVR               0b00100000 // prima di scrivere nel registro spostare >> 4
#define LPS25HB_INT_F_FTH               0b01000000 // prima di scrivere nel registro spostare >> 4
#define LPS25HB_INT_F_EMPTY             0b10000000 // prima di scrivere nel registro spostare >> 4

//LPS25HB_INTERRUPT_CFG
#define LPS25HB_PH_E                    0b00000001
#define LPS25HB_PL_E                    0b00000010
#define LPS25HB_LIR                     0b00000100

//LPS25HB_STATUS_REG
#define LPS25HB_T_DA                    0b00000001
#define LPS25HB_P_DA                    0b00000010

//LPS25HB_FIFO_CTRL
#define LPS25HB_FIFO_OFF                0b00000000
#define LPS25HB_F_MODE_BYPASS           0b00000000
#define LPS25HB_F_MODE_FIFO             0b00100000
#define LPS25HB_F_MODE_STREAM           0b01000000
#define LPS25HB_F_MODE_FIFO_MEAN        0b11000000
//#define LPS25HB_F_MODE_STREAM_2_FIFO    0b01100000
//#define LPS25HB_F_MODE_BYPASS_2_STREAM  0b10000000
//#define LPS25HB_F_MODE_BYPASS_2_FIFO    0b11100000
#define LPS25HB_FIFO_WTM_0              0b00000000
#define LPS25HB_FIFO_WTM_2              0b00000001
#define LPS25HB_FIFO_WTM_4              0b00000011
#define LPS25HB_FIFO_WTM_8              0b00000111
#define LPS25HB_FIFO_WTM_16             0b00001111    
#define LPS25HB_FIFO_WTM_32             0b00011111

//LPS25HB_FIFO_STATUS
#define LPS25HB_FIFO_STAT_FTH           0b10000000
#define LPS25HB_FIFO_STAT_OVR           0b01000000
#define LPS25HB_FIFO_STAT_EMPTY         0b00100000
#define LPS25HB_FIFO_STAT_FSS           0b00011111
#define LPS25HB_FIFO_STAT_ALL           0b11111111

#define ON                              1
#define OFF                             0


/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file LPS25HB.c
 * Libreria per la gestione del sensore di pressione ST LPS25HB via I2C
 * - Per un corretto funzionamento:
 *  + Includere la libreria I2C_init.c prima delle libreria LPS25HB.c
 *  + Impostare il modulo I2C da utilizzare per l'LPS25HB in LPS25HB_settings.h
 *  + Abilitare il modulo I2C utilizzato in I2C_init_settings.h
 *  + Avviare il modulo I2C nel programma principale richiamando la funzione I2C_init() o I2C2_init()
 * - Utilizzo:
 *  + Per utilizzare il sensore in modalit ONE SHOT:
 *   + Settare, se necessario, la risoluzione di acquisizione tramite la funzione LPS25HB_set_resolution()
     + Impostare il sensore in modalit ONE SHOT tramite la funzione LPS25HB_set_mode()
     + Abilitare il sensore tramite la funzione LPS25HB_power_mode()
     + Abilitare l'acquisizione dei dati di temperatura e pressione tramite la funzione LPS25HB_shot()
     + Leggere i dati tramite le apposite funzioni
     + Disabilitare il sensore, se necessario, tramite la funzione LPS25HB_power_mode()
 *  + Per utilizzare il sensore in modalit LETTURA CONTINUA:
 *   + Settare, se necessario, la risoluzione di acquisizione tramite la funzione LPS25HB_set_resolution()
     + Impostare la frequenza di acquisizione del dato tramite la funzione LPS25HB_set_mode()
     + Abilitare il sensore tramite la funzione LPS25HB_power_mode()
     + Impostare gli interrupt di ricezione del dato tramite apposita funzione oppure verificare la fine della conversione tramite le funzioni LPS25HB_check_press_data_ready() e LPS25HB_check_temp_data_ready()
     + Quando il dato  pronto leggere i dati tramite le apposite funzioni 
 * - Files:
 *  + LPS25HB.c: Codice funzioni 
 *  + LPS25HB.h: Parametri e prototipi
 *  + LPS25HB_settings.h: Impostazioni specifiche per progetto
 */

/**
 * Imposta la risoluzione della temperatura e della pressione
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _temp_res: settaggio per la risoluzione di acquisizione della temperatura (unsigned char):
 *              - LPS25HB_TEMP_RES_8
 *              - LPS25HB_TEMP_RES_16
 *              - LPS25HB_TEMP_RES_32
 *              - LPS25HB_TEMP_RES_64
 * @param   _press_res: settaggio per la risoluzione di acquisizione della pressione (unsigned char):
 *              - LPS25HB_PRES_RES_8
 *              - LPS25HB_PRES_RES_32
 *              - LPS25HB_PRES_RES_128
 *              - LPS25HB_PRES_RES_512
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char LPS25HB_set_resolution(unsigned char _dev_addr, unsigned char _temp_res, unsigned char _press_res);

/**
 * Imposta la modalit di funzionamento: one shot oppure con aggiornamento a varie frequenze
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _mode: seleziona la modalit di funzionamento (unsigned char)
 *              - LSP25HB_ODR_ONE_SHOT
 *              - LSP25HB_ODR_1HZ
 *              - LSP25HB_ODR_7HZ
 *              - LSP25HB_ODR_12HZ
 *              - LSP25HB_ODR_25HZ
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Sia in modalit one shot che in acquisizione continua abilitare il dispositivo tramite funzione LPS25HB_power_mode();   \n
 *          per funzionamento one shot abilitare l'acquisizione del dato tramite funzione LPS25HB_shot();
 */
signed char LPS25HB_set_mode(unsigned char _dev_addr, unsigned char _mode);

/**
 * Abilita o disabilita il sensore
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _mode: seleziona lo stato (unsigned char)
 *              - LSP25HB_POWER_DOWN
 *              - LSP25HB_ACTIVE
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char LPS25HB_power_mode(unsigned char _dev_addr, unsigned char _mode);

/**
 * Resetta i registri allo stato di avvio ed attende che il reset sia completo
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Bloccante se il sensore non funziona correttamente              
 */
signed char LPS25HB_sw_reset(unsigned char _dev_addr);

/**
 * Resetta la memoria interna ai valori di fabbrica ed attende che il reset sia completo
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Bloccante se il sensore non funziona correttamente              
 */
signed char LPS25HB_memory_reset(unsigned char _dev_addr);

/**
 * Avvia l'acquisizione dei dati ed attende che i dati siano pronti per essere letti
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Non  necessario verificare che il dato sia pronto tramite funzione LPS25HB_check_press_data_ready() o LPS25HB_check_temp_data_ready().   \n
 *          Bloccante se il sensore non funziona correttamente                 
 */
signed char LPS25HB_shot(unsigned char _dev_addr);

/**
 * Legge il valore dai 3 byte di registro della pressione e torna il valore grezzo
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  valore grezzo del registro della pressione (unsigned long)     
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)        
 */
unsigned long LPS25HB_read_press_raw(unsigned char _dev_addr);

/**
 * Converte il valore grezzo del registro della pressione in valore float
 * @param   _press: valore grezzo del registro della pressione (unsigned long)
 * @return  valore della pressione convertito (float)    
 * @note    Risoluzione massima 0,0002hPa      
 */
float LPS25HB_press_raw_2_float(unsigned long _press);

/**
 * Legge il valore dai 3 byte di registro della pressione e torna il valore il valore convertito in float
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  valore della pressione convertito (float)  
 * @note    Risoluzione massima 0,0002hPa    \n   
 *          Non torna eventuali errori di comunicazione I2C            
 */
float LPS25HB_read_press_float(unsigned char _dev_addr);

/**
 * Legge il valore dai 2 byte di registro della temperatura e torna il valore grezzo
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  valore grezzo del registro della temperatura (signed int)   
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)              
 */
signed int LPS25HB_read_temp_raw(unsigned char _dev_addr);

/**
 * Converte il valore grezzo del registro della temperatura in valore float
 * @param   _temp: valore grezzo del registro della temperatura (signed int)
 * @return  valore della temperatura convertito (float)
 * @note    Risoluzione massima 0,002C           
 */
float LPS25HB_temp_raw_2_float(signed int _temp);

/**
 * Legge il valore dai 2 byte di registro della temperatura e torna il valore il valore convertito in float
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  valore della temperatura convertito (float)
 * @note    Risoluzione massima 0,002C       \n
 *          Non torna eventuali errori di comunicazione I2C     
 */
float LPS25HB_read_temp_float(unsigned char _dev_addr);

/**
 * Abilita o disabilita la funzione Auto Zero. (La pressione attuale viene presa come valore di riferimento, i valori letti successivamente
 * vengono letti come variazione rispetto allo zero impostato).
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _on_off: auto zero (unsigned char)
 *              - ON
 *              - OFF
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char LPS25HB_auto_zero(unsigned char _dev_addr, unsigned char _on_off);

/**
 * Aggiunge o toglie un offset al dato di pressione letto prima di essere salvato nei registri PRESS_OUT
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _offset_float: valore da aggiungere o togliere dal dato letto (float)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char LPS25HB_set_onepoint_cal(unsigned char _dev_addr, float _offset_float);

/**
 * Controlla se il dato della pressione  pronto
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  
 *          - 0: dato pressione non pronto
 *          - 1: dato pressione pronto     
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Si setta quando un nuovo dato viene convertito e si resetta in automatico quando viene letto il dato 
 */
signed char LPS25HB_check_press_data_ready(unsigned char _dev_addr);

 /**
 * Controlla se il dato della temperatura  pronto
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  
 *          - 0: dato temperatura non pronto
 *          - 1: dato temperatura pronto     
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Si setta quando un nuovo dato viene convertito e si resetta in automatico quando viene letto il dato 
 */
signed char LPS25HB_check_temp_data_ready(unsigned char _dev_addr);

/**
 * Imposta la modalit FIFO e relativi parametri
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _fifo_mode: modalit FIFO (unsigned char)
 *              - LPS25HB_FIFO_OFF
 *              - LPS25HB_F_MODE_FIFO
 *              - LPS25HB_F_MODE_STREAM
 *              - LPS25HB_F_MODE_FIFO_MEAN
 * @param   _wtm: Watermark level (unsigned char)
 *              - LPS25HB_FIFO_WTM_0
 *              - LPS25HB_FIFO_WTM_2
 *              - LPS25HB_FIFO_WTM_4
 *              - LPS25HB_FIFO_WTM_8
 *              - LPS25HB_FIFO_WTM_16
 *              - LPS25HB_FIFO_WTM_32
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Implementati solo FIFO main mode
 */
signed char LPS25HB_set_fifo(unsigned char _dev_addr, unsigned char _fifo_mode, unsigned char _wtm);

/**
 * Legge il parametro desiderato dello stato del FIFO
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _to_check: parametro del registro FIFO_STATUS da controllare (unsigned char)
 *              - PS25HB_FIFO_STAT_FTH
 *              - LPS25HB_FIFO_STAT_OVR
 *              - LPS25HB_FIFO_STAT_EMPTY
 *              - LPS25HB_FIFO_STAT_FSS
 *              - LPS25HB_FIFO_STAT_ALL
 * @return  
 *          - Per LPS25HB_FIFO_STAT_FTH, LPS25HB_FIFO_STAT_OVR, LPS25HB_FIFO_STAT_EMPTY:
 *              - 1: stato verificato
 *              - 2: stato non verificato
 *          - Per LPS25HB_FIFO_STAT_FSS torna il valore dell'ultimo livello FIFO usato
 *          - Per LPS25HB_FIFO_STAT_ALL torna l'intero registro FIFO_STATUS
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
signed char LPS25HB_fifo_get_status(unsigned char _dev_addr, unsigned char _to_check);

/**
 * Imposta la modalit e attiva interrupt sul pin INT_DRDY. \n
 * In modalit DATA viene generato un interrupt quando  disponibile un nuovo dato di pressione oppure per lo stato FIFO selezionato \n
 * Nelle modalit di interrupt differenziali viene generato un interrupt a seconda delle impostazioni selezionate (pressione alta/pressione bassa rispetto ai valori di riferimento)
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   _source: sorgente interrupt (unsigned char)
 *              - LPS25HB_INT_DRDY
 *              - LPS25HB_INT_F_OVR
 *              - LPS25HB_INT_F_FTH
 *              - LPS25HB_INT_F_EMPTY
 *              - LPS25HB_INT_P_HIGH
 *              - LPS25HB_INT_P_LOW
 *              - LPS25HB_INT_P_HIGH_LOW
 * @param    _logic: logica interrupt (unsigned char)
 *              - LPS25HB_INT_ACTIVE_HIGH
 *              - LPS25HB_INT_ACTIVE_LOW
 * @param   _connection: collegamento hardware del pin INT_DRDY (unsigned char)
 *              - LPS25HB_INT_PUSH_PULL
 *              - LPS25HB_INT_OPEN_DRAIN
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Per utilizzare le funzioni di interrupt differenziali impostare i valori di riferimento e di threshold tramite la funzione LPS25HB_set_interrupt_ref()
 *          prima di impostare gli interrupt con la funzione LPS25HB_set_interrupt()
 */
signed char LPS25HB_set_interrupt(unsigned char _dev_addr, unsigned char _source, unsigned char _logic, unsigned char _connection);

/**
 * Imposta la pressione di riferimento e la threshold per far scattare l'interrupt
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @param   press: pressione di riferimento per l'interrupt in hPa (float)
 * @param   threshold: threshold di riferimento per l'interrupt in hPa (float)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Richiamare questa funzione prima della funzione LPS25HB_set_interrupt_ref()
 */
signed char LPS25HB_set_interrupt_ref(unsigned char _dev_addr, float _press, float _threshold);

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_check_interrupt_source
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      0: Nessun interrupt
 *              1: Interrupt pressione bassa 
 *              2: Interrupt pressione alta 
 *              3: Interrupt pressione alta e bassa
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Verifica l'evento differenziale che ha causato l'interrupt
 * ============================================================================================================================================ */

/**
 * Verifica l'evento differenziale che ha causato l'interrupt
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  
 *          - 0: Nessun interrupt
 *          - 1: Interrupt pressione bassa 
 *          - 2: Interrupt pressione alta 
 *          - 3: Interrupt pressione alta a basso     
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char LPS25HB_check_interrupt_source(unsigned char _dev_addr);

/**
 * Legge l'id del dispositivo (0xBD)
 * @param   _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * @return  Device ID (unsigned char)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned char  LPS25HB_read_device_id(unsigned char _dev_addr);
