/* ============================================================================================================================================= /
 * LIBRARY:         USART_init.c
 * FILE:            USART_init.h
 * LAST REV:        2.5
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         29/09/2013
 * UPDATED:         31/07/2017
 *
 * DESCRIP:         Parametri e prototipi per libreria USART_init.c
/* ============================================================================================================================================ */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#ifndef XTAL_FREQ
    #error Definire XTAL_FREQ
#endif

#if (XTAL_FREQ == 1)
    #define USART_9600_BRG16      1
    #define USART_9600_SPBRGH     0
    #define USART_9600_SPBRG      25
    #define USART_19200_BRG16     1
    #define USART_19200_SPBRGH    0
    #define USART_19200_SPBRG     12
//    #define USART_57600_BRG16     1
//    #define USART_57600_SPBRGH    0
//    #define USART_57600_SPBRG     34
//    #define USART_115200_BRG16    1
//    #define USART_115200_SPBRGH   0
//    #define USART_115200_SPBRG    16

#elif (XTAL_FREQ == 8)
    #define USART_9600_BRG16      0
    #define USART_9600_SPBRGH     0
    #define USART_9600_SPBRG      51
    #define USART_19200_BRG16     0
    #define USART_19200_SPBRGH    0
    #define USART_19200_SPBRG     25
    #define USART_57600_BRG16     1
    #define USART_57600_SPBRGH    0
    #define USART_57600_SPBRG     34
    #define USART_115200_BRG16    1
    #define USART_115200_SPBRGH   0
    #define USART_115200_SPBRG    16
    
#elif (XTAL_FREQ == 20)
    #define USART_9600_BRG16      1
    #define USART_9600_SPBRGH     2
    #define USART_9600_SPBRG      8
    #define USART_19200_BRG16     1
    #define USART_19200_SPBRGH    1
    #define USART_19200_SPBRG     3
    #define USART_57600_BRG16     1
    #define USART_57600_SPBRGH    0
    #define USART_57600_SPBRG     86
    #define USART_115200_BRG16    1
    #define USART_115200_SPBRGH   0
    #define USART_115200_SPBRG    42

#elif (XTAL_FREQ == 40)
    #define USART_9600_BRG16      1
    #define USART_9600_SPBRGH     4
    #define USART_9600_SPBRG      16
    #define USART_19200_BRG16     1
    #define USART_19200_SPBRGH    2
    #define USART_19200_SPBRG     8
    #define USART_57600_BRG16     1
    #define USART_57600_SPBRGH    0
    #define USART_57600_SPBRG     172
    #define USART_115200_BRG16    1
    #define USART_115200_SPBRGH   0
    #define USART_115200_SPBRG    86
    
#elif (XTAL_FREQ == 64)
    #define USART_9600_BRG16      1
    #define USART_9600_SPBRGH     6
    #define USART_9600_SPBRG      130
    #define USART_19200_BRG16     1
    #define USART_19200_SPBRGH    3
    #define USART_19200_SPBRG     64
    #define USART_57600_BRG16     1
    #define USART_57600_SPBRGH    1
    #define USART_57600_SPBRG     21
    #define USART_115200_BRG16    1
    #define USART_115200_SPBRGH   0
    #define USART_115200_SPBRG    138

#else
    #error inserire il valore di SPBRG per il nuovo valore di XTAL_FREQ 
#endif

#ifdef USART_MODULE_1                                                                   // definizione linee USART a seconda del modulo e del PIC usato
    #if defined (_18F4550) || defined (_18F2550) || defined (_18F46K22) || defined (_18F26K22) || defined (_18LF26K22)|| defined (_18F67K22)
        #define USART_TX_TRIS           TRISCbits.TRISC6
        #define USART_RX_TRIS           TRISCbits.TRISC7
        #define USART_INTERRUPT_IF      PIR1bits.RCIF
        #define USART_INTERRUPT_IE      PIE1bits.RCIE
        #define USART_INTERRUPT_IP      IPR1bits.RCIP
        
    #elif defined _12F1840
        #define USART_INTERRUPT_IF      PIR1bits.RCIF
        #define ESP8266_INTERRUPT_IE    PIE1bits.RCIE
    
        #if (USART_TX_RA == 0)
            #define USART_TX_TRIS       TRISAbits.TRISA0
        #elif (USART_TX_RA == 4)
            #define USART_TX_TRIS       TRISAbits.TRISA4
        #else
            #error USART_TX PIN non valido, impostare correttamente USART_TX_RA in USART_init_settings.h
        #endif
        #if (USART_RX_RA == 1)
            #define USART_RX_TRIS       TRISAbits.TRISA1
        #elif (USART_RX_RA == 5)
            #define USART_RX_TRIS       TRISAbits.TRISA5
        #else
            #error USART_RX PIN non valido, impostare correttamente USART_RX_RA in USART_init_settings.h
        #endif
    #else
        #error PIC non Gestito. Definire costanti PIN di uscita per il nuovo PIC in USART_init.h
    #endif
#endif

#ifdef USART_MODULE_2
    #define USART2_INTERRUPT_IF     PIR3bits.RC2IF
    #define USART2_INTERRUPT_IE     PIE3bits.RC2IE
    #define USART2_INTERRUPT_IP     IPR3bits.RC2IP

    #ifdef _18F46K22
        #define USART2_TX_TRIS TRISDbits.TRISD6
        #define USART2_RX_TRIS TRISDbits.TRISD7
    #elif defined (_18F26K22) || defined (_18LF26K22)
        #define USART2_TX_TRIS TRISBbits.TRISB6
        #define USART2_RX_TRIS TRISBbits.TRISB7
    #elif defined (_18F67K22)
        #define USART2_TX_TRIS TRISGbits.TRISG1
        #define USART2_RX_TRIS TRISGbits.TRISG2
    #elif defined (_18F4550) || defined (_18F2550) ||  defined (_12F1840)
        #error il PIC selezionato non ha il secondo modulo USART
    #else
        #error PIC non Gestito. Definire costanti PIN di uscita per il nuovo PIC in USART_init.h
    #endif
#endif

/* RINOMINAZIONE REGISTRI PIC18F46K22 ========================================================================================================= */
#if defined _18F46K22 || defined _18F67K22                                                                        // ridefinizione dei registri 
    #define TXSTAbits   TXSTA1bits
    #define RCSTAbits   RCSTA1bits
    #define SPBRG       SPBRG1
    #define SPBRGH      SPBRGH1
    #define TXREG       TXREG1
    #define RCIF        RC1IF
    #define RCREG       RCREG1
    #define BAUDCONbits BAUDCON1bits
    #define RCIE        RC1IE
    #define RCIP        RC1IP
#endif

/* COSTANTI =================================================================================================================================== */
#define IN  1
#define OUT 0

#define USART_BAUDR_9600    1
#define USART_BAUDR_19200   2
#define USART_BAUDR_57600   3
#define USART_BAUDR_115200  4

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file USART_init.c
 * Libreria per la gestione del bus USART primario e secondario (se disponibile).
 * - Per un corretto funzionamento:
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Impostare i moduli USART abilitati e le linee utilizzate in USART_init_settings.h
 *  + Impostare il modulo USART specifico per il dispositivo nei settaggi dei vari dispositivi (se il dispositivo funziona senza libreria specifica basta abilitare il modulo in USART_init_settings.h
 * - Files:
 *  + USART_init.c: Codice funzioni
 *  + USART_init.h: Parametri e prototipi
 *  + USART_init_settings.h: Impostazioni specifiche per progetto
 */

/**
 * Abilita l'invio e la ricezione dei dati via USART, Baud rate impostabile, 8 bit, senza parit . MODULO 1
 * @param   _baudr: settaggio baud rate (unsigned char)
 *              - USART_BAUDR_9600
 *              - USART_BAUDR_19200
 *              - USART_BAUDR_57600
 *              - USART_BAUDR_115200
 * @return  void
 */
void USART_enable(unsigned char _baudr);

/**
 * Abilita l'invio e la ricezione dei dati via USART, Baud rate impostabile, 8 bit, senza parit . MODULO 2
 * @param   _baudr: settaggio baud rate (unsigned char)
 *              - BAUDR_9600
 *              - BAUDR_19200
 *              - BAUDR_57600
 *              - BAUDR_115200
 * @return  void
 */
void USART2_enable(unsigned char _baudr);

/**
 * Invia un byte via USART e attende il flag di invio completato. MODULO 1
 * @param   _data: byte da inviare (unsigned char)
 * @return  void
 */
void USART_send_char(unsigned char _data);

/**
 * Invia un byte via USART e attende il flag di invio completato. MODULO 2
 * @param   _data: byte da inviare (unsigned char)
 * @return  void
 */
void USART2_send_char(unsigned char _data);

/**
 * Invia una stringa via USART. MODULO 1
 * @param   _string[]: Stringa da inviare
 * @return  void
 */
void USART_send_string(const unsigned char _string[]);

/**
 * Invia una stringa via USART. MODULO 2
 * @param   _string[]: Stringa da inviare
 * @return  void
 */
void USART2_send_string(const unsigned char _string[]);

/**
 * Invia una stringa di dati di linghezza fissa via USART. MODULO 1 
 * @param   _string[]: Stringa da inviare
 * @param   _n: numero di byte da inviare (unsigned char)
 * @return  void
 * @note    A differenza della funzione USART_send_string non si ferma se il byte da inviare  0x00 (carattere di fine stringa ('\0')
 */
void USART_send_n_chars(const unsigned char _string[], unsigned char _n);

/**
 * Invia una stringa di dati di linghezza fissa via USART. MODULO 2
 * @param   _string[]: Stringa da inviare
 * @param   _n: numero di byte da inviare (unsigned char)
 * @return  void
 * @note    A differenza della funzione USART_send_string non si ferma se il byte da inviare  0x00 (carattere di fine stringa ('\0')
 */
void USART_send_n_chars(const unsigned char _string[], unsigned char _n);

/**
 * Restituisce il byte ricevuto, bloccante se non si riceve nulla. MODULO 1
 * @param   void
 * @return  byte ricevuto (unsigned char)
 */
unsigned char USART_receive(void);

/**
 * Restituisce il byte ricevuto, bloccante se non si riceve nulla. MODULO 2
 * @param   void
 * @return  byte ricevuto (unsigned char)
 */
unsigned char USART2_receive(void);

