/* ============================================================================================================================================= /
 * LIBRARY:         ESP8266
 * FILE:            ESP8266_settings.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         10/01/2017
 * UPDATED:         
 *
 * DESCRIP:         Settaggi per ESP8266.c
/* ============================================================================================================================================ */

/* Definire su quale modulo USART  collegato il dispositivo */
#define ESP8266_USART_MODULE_1
//#define ESP8266_USART_MODULE_2

/* Definire il livello di interrupt del modulo USART */
#define ESP8266_INT_PRIORITY 1                                                          // interrupt bassa priorit
#define ESP8266_TIMER_INT_PRIORITY 1

// Impostazioni Debug
//#define DEBUG_RESEND_ON_UART_RAW                                                      // Possibili errori se baudrate ESP8266  impostato a 115200
//#define DEBUG_RESEND_ON_UART_CLEANED
//#define DEBUG_ON_UART_1

/* Impostazioni PIN dispositivo */
#define ESP8266_CHPD    LATCbits.LATC2
#define ESP8266_RST     LATCbits.LATC5

/* Impostazioni dispositivo */
#define ESP8266_BUFFER_LENGTH   200    // 500 a 115200 baud 200 a 9600
#define ESP8266_DEF_TIMEOUT     500    // ms timeout default
#define ESP8266_SEND_TIMEOUT    3000   // ms timeout per operazioni di invio dati    

/* Impostazioni timer, definire i registri e modificare la funzione di inizializzazione a seconda del PIC/timer usato */
#define ESP8266_TIMER_ON              T1CONbits.TMR1ON
#define ESP8266_TIMER_INT_IE          PIE1bits.TMR1IE
#define ESP8266_TIMER_INT_IF          PIR1bits.TMR1IF
#define ESP8266_TIMER_INT_IP          IPR1bits.TMR1IP
#define ESP8266_TMR                   TMR1
#define ESP8266_TIMER_PRELOAD         45536

// TIMER 1, Configurazioni per avere 1 interrupt ogni 5ms = (65536-preload)*(4/fosc)*prescaler = (65536-45536)*(4/64000000)*4
void ESP8266_set_timer(void){
    T1CONbits.T1CKPS0 = 0;                                                              // prescaler 1/4
    T1CONbits.T1CKPS1 = 1;
    T1CONbits.TMR1CS = 0;                                                               // da clock interno fosc/4
    T1CONbits.RD16 = 1;                                                                 // 16 BIT
    ESP8266_TMR = ESP8266_TIMER_PRELOAD;                                                // preload timer

    ESP8266_TIMER_ON = OFF;                                                             // timer spento

    ESP8266_TIMER_INT_IP = ESP8266_TIMER_INT_PRIORITY;                                  
    ESP8266_TIMER_INT_IF= 0;                                                            // priorit interrupt timer
    ESP8266_TIMER_INT_IE= ON;                                                           // abilito interrupt timer 1 
}