/* ============================================================================================================================================= /
 * LIBRARY:         ESP8266
 * FILE:            ESP8266.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         10/01/2017
 * UPDATED:         14/02/2017
 *
 * DESCRIP:         Parametri e prototipi per libreria ESP8266.c
 * ============================================================================================================================================= /
 * Change History:
 * Rev.              Date                Changes
 * 1.0               14/02/2017          - Prima build completa
/* ============================================================================================================================================ */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#ifndef USART                     
    #error Includere USART_init.c
#endif

#ifdef ESP8266_USART_MODULE_1                                                           // Ridefinisco le chiamate alle funzioni a seconda del modulo USART usato
    #ifndef USART_MODULE_1
        #error USART_MODULE_1 non abilitato in USART_init_settings.h
    #endif
    #define _ESP8266_USART_send_string  USART_send_string
    #define _ESP8266_USART_send_char    USART_send_char
    #define _ESP8266_USART_send_n_chars USART_send_n_chars
    #define _ESP8266_USART_receive      USART_receive
    #define _ESP8266_USART_enable       USART_enable
    #define ESP8266_INTERRUPT_IF        USART_INTERRUPT_IF
    #define ESP8266_INTERRUPT_IE        USART_INTERRUPT_IE
    #define ESP8266_INTERRUPT_IP        USART_INTERRUPT_IP
    
#elif defined ESP8266_USART_MODULE_2
    #ifndef USART_MODULE_2
        #error USART_MODULE_2 non abilitato in USART_init_settings.h
    #endif
    #define _ESP8266_USART_send_string  USART2_send_string
    #define _ESP8266_USART_send_char    USART2_send_char
    #define _ESP8266_USART_send_n_chars USART2_send_n_chars
    #define _ESP8266_USART_receive      USART2_receive
    #define _ESP8266_USART_enable       USART2_enable
    #define ESP8266_INTERRUPT_IF        USART2_INTERRUPT_IF
    #define ESP8266_INTERRUPT_IE        USART2_INTERRUPT_IE
    #define ESP8266_INTERRUPT_IP        USART2_INTERRUPT_IP

#else
    #error Definire il modulo USART da utilizzare per ESP8266_settings.h
#endif

// impostazioni di debug per resend dei dati ricevuti su porta UART secondaria
#ifdef DEBUG_ON_UART_1
    #ifndef USART_MODULE_1
        #error USART_MODULE_1 non abilitato in USART_init_settings.h
    #endif
    #define _ESP8266_deb_USART_send_string  USART_send_string
    #define _ESP8266_deb_USART_send_char    USART_send_char
#elif defined DEBUG_ON_UART_2  
    #ifndef USART_MODULE_2
        #error USART_MODULE_2 non abilitato in USART_init_settings.h
    #endif  
    #define _ESP8266_deb_USART_send_string  USART2_send_string
    #define _ESP8266_deb_USART_send_char    USART2_send_char
#endif

/* COSTANTI =================================================================================================================================== */
#define ON  1
#define OFF 0

#define ESP8266_BAUDR_9600      1
#define ESP8266_BAUDR_19200     2
#define ESP8266_BAUDR_57600     3
#define ESP8266_BAUDR_115200    4

#define ESP8266_CURRENT_VALUE   0
#define ESP8266_DEFAULT_VALUE   1

#define ESP8266_PARITY_NONE     0
#define ESP8266_PARITY_ODD      1
#define ESP8266_PARITY_EVEN     2

#define ESP8266_FLOW_CTRL_NONE  0
#define ESP8266_FLOW_CTRL_RTS   1
#define ESP8266_FLOW_CTRL_CTS   2
#define ESP8266_FLOW_CTRL_BOTH  3


/* RISPOSTE DA CERCARE NEL BUFFER DI RITORNO
 * Per aggiungere risposte: 
 * - inserire la stringa in RESPONSES_2
 * - aggiungere la lunghezza della nuova risposta in RESP_LENGTH_2
 * - incrementeare RESP_HANDLED_2
 * - adattare RESP_MAX_LENGTH_2 alla lunghezza massima della risposta da interpretare
 * - definire la nuova risposta con un numero incrementale
 * ATTENZIONE, SE UNA RISPOSTA CONTIENE ALL'INIZIO GLI STESSI CARATTERI COMPLETI DI UNA RISPOSTA PRECEDENTE, VERRA' PRESA LA PRIMA COME RISPOSTA VERIFICATA
 * METTERE PRIMA LA RISPOSTA PI LUNGA
 */

//                        0        1     2         3         4          5           6           7           8         9       10        11        12        13          14         15          16         17         18        19     20     21     22     23     24                             
#define RESPONSES_1       "ERROR", "OK", "ATE0OK", "AT ver", "+CWMODE", "WIFI CON", "WIFI DIS", "WIFI GOT", "+CWJAP", "NoAP", "+CWLAP", "+CWSAP", "+CIFSR", "+CWDHCPS", "+CWDHCP", "+CIPSTAM", "+CIPAPM", "+CIPSTA", "+CIPAP", "0,C", "1,C", "2,C", "3,C", "4,C", "CONNECT F"
#define RESP_HANDLED_1    25
#define RESP_MAX_LENGTH_1 10 

//                          0         1        2        3    4         5         6         7         8         9         10        11        12        13   14        15
#define RESPONSES_2         "CONNEC", "CLOSE", "+IPD,", ">", "+CIPST", "SEND O", "SEND F", "linkis", "+CIPDO", "ALREAD", "DNS Fa", "UNLINK", "+timeo", "+", "STATUS", "busy"
#define RESP_HANDLED_2      16
#define RESP_MAX_LENGTH_2   7     
 
/* RISPOSTE INCREMENTALI */
#define ESP8266_RESP_ERROR          0x00
#define ESP8266_RESP_OK             0x01
#define ESP8266_RESP_ATE0_OK        0x02
#define ESP8266_RESP_ATVER          0x03
#define ESP8266_RESP_CWMODE         0x04
#define EPS8266_RESP_WIFI_CONNEC    0x05
#define EPS8266_RESP_WIFI_DISCON    0x06
#define EPS8266_RESP_WIFI_GOT_IP    0x07
#define EPS8266_RESP_CWJAP          0x08
#define ESP8266_RESP_NOAP           0x09
#define EPS8266_RESP_CWLAP          0x0A
#define EPS8266_RESP_CWSAP          0x0B
#define ESP8266_RESP_CIFSR          0x0C
#define ESP8266_RESP_CWDHCPS        0x0D
#define ESP8266_RESP_CWDHCP         0x0E
#define ESP8266_RESP_CIPSTAMAC      0x0F
#define ESP8266_RESP_CIPAPMAC       0x10
#define ESP8266_RESP_CIPSTA         0x11
#define ESP8266_RESP_CIPAP          0x12

#define ESP8266_RESP_CON_0          0x13
#define ESP8266_RESP_CON_1          0x14
#define ESP8266_RESP_CON_2          0x15
#define ESP8266_RESP_CON_3          0x16
#define ESP8266_RESP_CON_4          0x17
#define ESP8266_RESP_CONNECT_FAIL   0x18
#define ESP8266_RESP_CONNECT        0x19
#define ESP8266_RESP_CLOSE          0x1A

#define ESP8266_RESP_IPD            0x1B
#define ESP8266_RESP_WRAP_RET       0x1C
#define ESP8266_RESP_CIPSTO         0x1D
#define ESP8266_RESP_SEND_OK        0x1E
#define ESP8266_RESP_SEND_FAIL      0x1F
#define ESP8266_RESP_LINK_NOT_VALID 0x20
#define ESP8266_RESP_CIPDOMAIN      0x21
#define ESP8266_RESP_ALREADY_CON    0x22
#define ESP8266_RESP_DNS_FAIL       0x23
#define ESP8266_RESP_UNLINK         0x24  
#define ESP8266_RESP_PING_TO        0x25
#define ESP8266_RESP_PING           0x26
#define ESP8266_RESP_STATUS         0x27
#define ESP8266_RESP_BUSY           0x28

#define ESP8266_CON_FAIL_OFFSET     0x30
#define ESP8266_CON_0_FAIL          0x30
#define ESP8266_CON_1_FAIL          0x31
#define ESP8266_CON_2_FAIL          0x32
#define ESP8266_CON_3_FAIL          0x33
#define ESP8266_CON_4_FAIL          0x34

#define ESP8266_CON_CONNECT_OFFSET  0x40
#define ESP8266_CON_0_CONNECT       0x40
#define ESP8266_CON_1_CONNECT       0x41
#define ESP8266_CON_2_CONNECT       0x42
#define ESP8266_CON_3_CONNECT       0x43
#define ESP8266_CON_4_CONNECT       0x44

#define ESP8266_CON_CLOSE_OFFSET    0x50
#define ESP8266_CON_0_CLOSE         0x50
#define ESP8266_CON_1_CLOSE         0x51
#define ESP8266_CON_2_CLOSE         0x52
#define ESP8266_CON_3_CLOSE         0x53
#define ESP8266_CON_4_CLOSE         0x54

#define ESP8266_CON_SEND_OFFSET     0x60
#define ESP8266_CON_0_SEND          0x60
#define ESP8266_CON_1_SEND          0x61
#define ESP8266_CON_2_SEND          0x62
#define ESP8266_CON_3_SEND          0x63
#define ESP8266_CON_4_SEND          0x64

#define ESP8266_CWJAP_OFFSET        0x80
#define ESP8266_TIMEOUTCON          0x81
#define ESP8266_PWD_ERR             0x82
#define ESP8266_NO_AP               0x83
#define ESP8266_CON_FAIL            0x84

#define ESP8266_CWMODE_OFFSET       0x90
#define ESP8266_STATION_MODE        0x91
#define ESP8266_SOFTAP_MODE         0x92
#define ESP8266_BOTH_MODE           0x93

#define ESP8266_DHCP_OFFSET         0xA0
#define ESP8266_DHCP_AP             0xA1
#define ESP8266_DHCP_STATION        0xA2
#define ESP8266_DHCP_BOTH           0xA3

#define ESP8266_ENC_OFFSET          0xB0
#define ESP8266_ENC_OPEN            0xB0
#define ESP8266_ENC_WEP             0xB1
#define ESP8266_ENC_WPA_PSK         0xB2
#define ESP8266_ENC_WPA2_PSK        0xB3
#define ESP8266_ENC_WPA_WPA2_PSK    0xB4
#define ESP8266_ENC_WPA2_ENT        0xB5

#define ESP8266_STAUTS_OFFSET       0xC0
#define ESP8266_CON_AP_AND_IP       0xC2
#define ESP8266_STA_TCP_UDP_TRA     0xC3
#define ESP8266_STA_TCP_UDP_TRA_DIS 0xC4
#define ESP8266_STA_NOT_CON_AP      0xC5    

#define ESP8266_NO_INCOMING_DATA    0xF0 
#define ESP8266_NO_STATION_CON      0xFA
#define ESP8266_RESP_RECEIVED       0xFB
#define ESP8266_NOT_READY           0xFC    
#define ESP8266_READY               0xFD
#define ESP8266_RESP_TIMEOUT        0xFE
#define ESP8266_RESP_NOT_HANDLED    0xFF  

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file ESP8266.c
 * Libreria per la gestione del modulo WiFi ESP8266 via USART.
 * - Per un corretto funzionamento:
 *  + Includere la libreria Delay.c prima della libreria ESP8266.c
 *  + Includere la libreria USART_init.c prima della libreria ESP8266.c 
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Impostare il modulo USART da utilizzare per il dispositivo ESP8266 in ESP8266_settings.h
 *  + Impostare i registri del timer utilizzato in ESP8266_settings.h
 *  + Abilitare il modulo USART utilizzato in USART_init_settings.h
 *  + In main impostare il modulo usart tramite funzione USART_eneble(BAUDRATE) ed inizializzare il modulo con la funzione ESP8266_init()
 *  + Abilitare gli interrupt globali con priorit
 *  + Prima di accedere ad un access point o prima di settare un soft access point impostare la modalit (attuale o di default) tramite la funzione
 *    ESP8266_set_mode()
 *  + Per verificare trasmissioni di dati in ingresso, non provenienti da risposte alle funzioni, verificare in polling la funzione ESP8266_check_incoming_data() 
 *    in questo caso resettare il buffer tramite la funzione ESP8266_reset_buffer() dopo ogni stringa rcevuta ed interpretata 
 * - Files:
 *  + ESP8266.c: Codice funzioni
 *  + ESP8266.h: Parametri e prototipi
 *  + ESP8266_settings.h impostazioni specifiche per progetto
 */

/** 
 * Imposta le linee di ricezione, abilita e resetta il modulo e disabilita l'echo automatico.
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)   
 *          - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_init(unsigned int _timeout);

/** 
 * Invia comando di reset, verifica il corretto riavvio del modulo ESP8266 e disabilita l'echo automatico.
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)   
 *          - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_soft_reset(unsigned int _timeout);

/** 
 * Resetta tramite PIN CH_PD il modulo ESP8266, verifica il corretto riavvio e disabilita l'echo automatico.
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)   
 *          - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_hard_reset(unsigned int _timeout);

/** 
 * Spegne o accende tramite PIN CH_PD il modulo ESP8266, verifica il corretto avvio e disabilita l'echo automatico.
 * @param   _on_off: stato da impostare: ON/OFF (unsigned char)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)   
 *          - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_power(unsigned char _on_off, unsigned int _timeout);

/** 
 * Invia comando AT e verifica la risposta
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (il modulo ESP8266 ha restituito OK)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_at(unsigned int _timeout);

/** 
 * Invia comando di lettura versione firmware del modulo. Torna 1 se i dati sono ricevuti correttamente.
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (firmware letto correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    In esp8266_buffer torna i dati letti        \n
 *          ATversion:1.3.0.0(Jul14201618:54:01)SDKversion:2.0.0(656edbf)compiletime:Jul19201618:43:55OK          
 */                          
unsigned char ESP8266_read_firmware(unsigned int _timeout);

/** 
 * Attiva o disattiva l'echo automatico da parte del modulo ESP8266
 * @param   _on_off: stato da impostare: ON/OFF (unsigned char)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0x02: ESP8266_RESP_ATE0_OK (ATE0 impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    Per ridurre il buffer di ricezione La libreria funziona con echo disattivato, l'unico comando interpretato con echo attivato 
 *          ATE0 per disabilitare l'echo automatico.         
 */ 
unsigned char ESP8266_echo(unsigned char _on_off, unsigned int _timeout); 

/** 
 * Invia comando di ripristino impostazioni di programmazione (che non sono quelle programmate con funzioni DEF) reimposta la modalit di trasmissione
 * del modulo UART e reinizializza il modulo ESP8266
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 1000ms) (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x02: ESP8266_RESP_ATE0_OK (impostazioni ripristinate e modulo riavviato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_restore_factory(unsigned int _timeout);             

/** 
 * Imposta la modalit di trasmissione UART (attuale o default) e reimposta la velocit di trasmissione del modulo UART
 * @param   _baud_r: velocit di trasmissione (unsigned char):
 *              - ESP8266_BAUDR_9600
 *              - ESP8266_BAUDR_19200
 *              - ESP8266_BAUDR_57600
 *              - ESP8266_BAUDR_115200  
 * @param   _data_bits: numero di bit della trasmissione: 5~8 (unsigned char)
 * @param   _stop_bits: numero di bit di stop (unsigned char): 
 *              - 1:1
 *              - 2:1.5
 *              - 3:2 
 * @param   _parity: impostazione check parit (unsigned char):
 *              - ESP8266_PARITY_NONE
 *              - ESP8266_PARITY_ODD
 *              - ESP8266_PARITY_EVEN      
 * @param   _flow_ctrl: impostazione controllo flusso (unsigned char): 
 *              - ESP8266_FLOW_CTRL_NONE
 *              - ESP8266_FLOW_CTRL_RTS
 *              - ESP8266_FLOW_CTRL_CTS
 *              - ESP8266_FLOW_CTRL_BOTH        
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default     
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametri impostati correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)     
 */    
unsigned char ESP8266_uart_conf(unsigned char _baud_r, unsigned char _data_bits, unsigned char _stop_bits, unsigned char _parity, unsigned char _flow_ctrl, unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta la potenza di trasmissione
 * @param   _rf_power[]: potenza di trasmissione: stringa "0"~"82" a passi di 0.25dBm per unit (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)     
 */    
unsigned char ESP8266_set_tx_power(const unsigned char _rf_power[], unsigned int _timeout);

/** 
 * Imposta la modalit di funzionamento del modulo (attuale o default)
 * @param   _mode: modalit di funzionamento (unsigned char):
 *              - ESP8266_STATION_MODE
 *              - ESP8266_SOFTAP_MODE
 *              - ESP8266_BOTH_MODE 
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)     
 */  
unsigned char ESP8266_set_mode(unsigned char _mode,  unsigned char _cur_def, unsigned int _timeout);

/** 
 * legge la modalit di funzionamento del modulo (attuale o default)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x91: ESP8266_STATION_MODE (modulo ESP8266 in modalit Stazione)
 *          - 0x92: ESP8266_SOFTAP_MODE (modulo ESP8266 in modalit Soft Access Point)
 *          - 0x93: ESP8266_SOFTAP_MODE (modulo ESP8266 in modalit Stazione + Soft Access Point)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)     
 */
unsigned char ESP8266_read_mode(unsigned char _cur_def, unsigned int _timeout);        
 
/** 
 * Accede alla rete WiFi
 * @param   _ssid[]: nome della rete (stringa)
 * @param   _password[]: password (stringa)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: Rete connessa e IP assegnato
 *          - 0x05: EPS8266_RESP_WIFI_CONNEC (WiFi connesso ma IP non assegnato)
 *          - 0x07: EPS8266_RESP_WIFI_GOT_IP (WiFi indirizzo IP assegnato)
 *          - 0x81: ESP8266_TIMEOUTCON (timeout connessione)
 *          - 0x82: ESP8266_PWD_ERR (password errata)
 *          - 0x83: ESP8266_NO_AP (access Point non trovato)
 *          - 0x84: ESP8266_CON_FAIL (Connessione fallita)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 *          - FUNZIONAMENTO CON DHCP:
 *              - Abilitare il DHCP sul modulo (impostazione di default) e nell'access point 
 *          - FUNZIONAMENTO CON IP STATICO
 *              - Disabilitare il DHCP (ESP8266_set_dhcp())
 *              - Assegnare un indirizzo IP valido (ESP8266_set_IP())     
 */
unsigned char ESP8266_join_ap(const unsigned char _ssid[], const unsigned char _password[], unsigned char _cur_def, unsigned int _timeout);  

/** 
 * Legge access point connesso e torna 1 se i dati sono ricevuti correttamente.
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: Access point connesso
 *          - 0x09: EPS8266_RESP_NOAP (nessun Access Point connesso)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          Se connesso: "Home","e0:91:f5:4d:0c:3c",10,-49OK       \n
 *          Se non connesso: NoAPOK
 */ 
unsigned char ESP8266_check_ap_con(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta i parametri da restituire durante la scansione degli access point (ESP8266_search_ap())
 * @param   _on_off: ordinamento in base alla potenza del segnale: ON/OFF (unsigned char)
 * @param   _mask[]: stringa di mascheramento dei risultati ("31" per mostrare solo ecn, ssid, rssi, mac, ch) (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    l'impostazione si resetta al riavvio del modulo
 */ 
unsigned char ESP8266_search_ap_opt(unsigned char _on_off, const unsigned char _mask[], unsigned int _timeout);

/** 
 * Legge gli access point nelle vicinanze. Torna 1 se ha finito e ha trovato access point.
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (AP trovati)
 *          - 0x09: ESP8266_RESP_NOAP (nessun AP trovato)) 
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere  impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          +CWLAP:(1,"Home",-36,"e0:91:f5:4d:0c:3c",10)+CWLAP:(1,"Home2",-75,"4c:9e:ff:04:5a:48",3       \n
 *          +CWLAP:(4,"Alice-49348481",-80,"d4:d1:84:30:cc:38",13)OK)       \n
 */ 
unsigned char ESP8266_search_ap(unsigned int _timeout);

/** 
 * Disconnette dalla rete attualmente connessa
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x06: EPS8266_RESP_WIFI_DISCON (WiFi disconnesso)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_quit_ap(unsigned int _timeout);       

/** 
 * Imposta il modulo ESP8266 come Access Point
 * @param   _ssid[]: nome della rete (stringa)
 * @param   _password[]: password (stringa)
 * @param   _channel[]: canale della trasmissione (stringa)
 * @param   _enc: modi di criptazione(unsigned char) :
 *              - ESP8266_ENC_OPEN
 *              - ESP8266_ENC_WPA_PSK
 *              - ESP8266_ENC_WPA2_PSK
 *              - ESP8266_ENC_WPA_WPA2_PSK      
 * @param   _max_con: numero di connessioni massime (1~4) (unsigned char)
 * @param   _hidden: ssid nascosto (ON/OFF) (unsigned char)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (soft Accesso Point impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre    Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 *         - FUNZIONAMENTO CON DHCP:
 *              - Abilitare il DHCP sul modulo (impostazione di default) e nell'access point 
 *              - Definire eventualmente l'intervallo di indirizzi che il DHCP pu assegnare (funzione settaggio indirizzi DHCP)
 *         - FUNZIONAMENTO CON IP STATICO
 *              - Disabilitare il DHCP (ESP8266_set_dhcp())
 *              - Assegnare un indirizzo IP valido al client da connettere al modulo ESP8266
 * @note    WEP non supportato              
 */
unsigned char ESP8266_set_softap(const unsigned char _ssid[], const unsigned char _password[], const unsigned char _channel[], unsigned char _enc, unsigned char _max_con, unsigned char _hidden, unsigned char _cur_def, unsigned int _timeout);

/** 
 * legge i parametri del soft Access Point
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (dati soft Access Point letti correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 * @note    In esp8266_buffer torna i dati letti:
 *          es: "TEST","123456789",5,2,4,0              
 */
unsigned char ESP8266_read_softap(unsigned char _cur_def, unsigned int _timeout);

/** 
 * legge gli indirizzi IP e MAC di eventuali stazioni connesse al Soft AP
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (stazioni connesse lette correttamente)
 *          - 0xFA: ESP8266_NO_STATION_CON (nessuna stazione connessa)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 * @note    In esp8266_buffer torna i dati letti:
 *          es: 192.168.4.2,90:18:7c:25:aa:db  OK             
 */
unsigned char ESP8266_read_connected_stations(unsigned int _timeout);

/** 
 * Attiva o disattiva il DHCP per la modalit di funzionamento selezionata (attuale o default)
 * @param   _mode: modalit di funzionamento sulla quale abilitare o disabilitare il DHCP: ESP8266_DHCP_AP/ESP8266_DHCP_STATION/ESP8266_DHCP_BOTH (unsigned char)
 * @param   _on_off: ON/OFF (unsigned char)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametri DHCP impostati correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Per abilitare o disabilitare il DHCP il modulo ESP8266 deve avere la relativa modalit abilitata (ESP8266_set_mode())
 * @note    
 *          - Di default il DHCP  abilitato per entrambe le modalit di funzionamento   
 *          - Pu essere impostato il DHCP solo per la modalit selezionata (ESP8266_set_mod())
 */ 
unsigned char ESP8266_set_dhcp(unsigned char _mode, unsigned char _on_off, unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge lo stato del DHCP per tutti i modi di funzionamento (attuale o default)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0xA0: DHCP disabilitato per Soft Access Point e Stazione
 *          - 0xA1: DHCP abilitato per Soft Access Point
 *          - 0xA2: DHCP abilitato per Stazione
 *          - 0xA3: DHCP abilitato per Soft Access Point e Stazione
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_read_dhcp(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta il range di IP che il DHCP pu assegnare
 * @param   _lease_time[]: tempo di mantenimento ip in minuti, max 2880 (stringa)
 * @param   _start_ip[]: indirizzo IP di partenza (stringa)
 * @param   _end_ip[]: indirizzo IP finale (stringa)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):          
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzi DHCP impostati correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     
 *          - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())   
 *          - Il modulo ESP8266 deve avere il DHCP per la modalit Soft AP abilitato (ESP8266_set_dhcp()    
 * @note    Gli indirizzi ip devono essere sullo stesso segmento di rete dell'ip assegnato alla modalita Soft Access Point         
 */ 
unsigned char ESP8266_set_dhcp_ip_range(const unsigned char _lease_time[], const unsigned char _start_ip[], const unsigned char _end_ip[],  unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta il range di IP di default che il DHCP pu assegnare
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):          
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzi DHCP impostati correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     
 *          - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())   
 *          - Il modulo ESP8266 deve avere il DHCP per la modalit Soft AP abilitato (ESP8266_set_dhcp()  
 */ 
unsigned char ESP8266_set_dhcp_ip_range_default(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge gli access point nelle vicinanze. Torna 1 se ha finito e ha trovato access point.
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):          
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzi DHCP letti correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     
 *          - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())   
 *          - Il modulo ESP8266 deve avere il DHCP per la modalit Soft AP abilitato (ESP8266_set_dhcp()  
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          es: 60,192.168.4.10,192.168.4.50OK          
 */ 
unsigned char ESP8266_read_dhcp_ip_range(unsigned char _cur_def, unsigned int _timeout); 

/** 
 * Imposta l'autoconnessione all'avvio
 * @param   _on_off: ON/OFF (unsigned char)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 * @note    Impostazione salvata e mantenuta al riavvio
 */ 
unsigned char ESP8266_auto_con(unsigned char _on_off, unsigned int _timeout);   

/** 
 * Imposta l'indirizzo MAC della modalit stazione(attuale o default)
 * @param   _mac[]: Indirizzo MAC da impostare (stringa) 
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo MAC stazione impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)    
 * @pre     Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())     
 * @note    
 *          - l'indirizzo MAC della modalit stazione e della modalit Soft Access Point devono essere diversi
 *          - il bit 0 del primo byte deve essere 0 (es: 1A va bene, 15 no)         
 */ 
unsigned char ESP8266_set_mac_sta(const unsigned char _mac[], unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge l'indirizzo MAC della modalit stazione(attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo MAC stazione letto correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    In esp8266_buffer torna i dati letti:
 *          es: "5c:cf:7f:8b:5c:d0"OK        
 */ 
unsigned char ESP8266_read_mac_sta(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta l'indirizzo MAC della modalit Soft Access Point (attuale o default)
 * @param   _mac[]: Indirizzo MAC da impostare (stringa) 
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo MAC Soft Access Point impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())  
 * @note    
 *          - l'indirizzo MAC della modalit stazione e della modalit Soft Access Point devono essere diversi
 *          - il bit 0 del primo byte deve essere 0 (es: 1A va bene, 15 no)            
 */ 
unsigned char ESP8266_set_mac_ap(const unsigned char _mac[], unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge l'indirizzo MAC della modalit Soft Access Point (attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo MAC Soft Access Point letto correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    In esp8266_buffer torna i dati letti:
 *          es: "5c:cf:7f:8b:5c:d0"OK      
 */ 
unsigned char ESP8266_read_mac_ap(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta l'indirizzo IP della modalit stazione(attuale o default)
 * @param   _ip[]: Indirizzo IP da impostare (stringa) 
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo IP Stazione impostato correttamente)    
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il DHCP per la modalit stazione deve essere disattivato (ESP8266_set_dhcp)         
 */ 
unsigned char ESP8266_set_ip_sta(const unsigned char _ip[], unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge l'indirizzo IP, l'indirizzo Gateway e la Subnet mask della modalit stazione(attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo IP Stazione letto correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     
 *          - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP(ESP8266_set_mode())
 *          - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          +CIPSTA_CUR:ip:"192.168.0.50"       \n
 *          +CIPSTA_CUR:gateway:"192.168.0.1"       \n
 *          +CIPSTA_CUR:netmask:"255.255.255.0"          
 */ 
unsigned char ESP8266_read_ip_sta(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Imposta l'indirizzo IP e l'indirizzo Gateway della modalit Soft Access Point (attuale o default)
 * @param   _ip[]: Indirizzo IP da impostare (stringa) 
 * @param   _gateway[]: indirizzo IP gateway (stringa)
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo IP Soft Access Point impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */ 
unsigned char ESP8266_set_ip_ap(const unsigned char _ip[], const unsigned char _gateway[], unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge l'indirizzo IP, l'indirizzo Gateway e la Subnet mask della modalit Soft Access Point (attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * @param   _cur_def: impostazione di salvataggio:  (unsigned char):
 *              - ESP8266_CURRENT_VALUE: imposta il valore attuale
 *              - ESP8266_DEFAULT_VALUE: imposta il valore di default  
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzo IP Soft Access Point letto correttamente)    
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostata in modalit Soft AP (ESP8266_set_mode())
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          +CIPAP_CUR:ip:"192.168.0.50"       \n
 *          +CIPAP_CUR:gateway:"192.168.0.1"       \n
 *          +CIPAP_CUR:netmask:"255.255.255.0"        
 */ 
unsigned char ESP8266_read_ip_ap(unsigned char _cur_def, unsigned int _timeout);

/** 
 * Legge indirizzo IP e MAC di stazione e soft ap. Torna 1 se i dati sono ricevuti correttamente
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (indirizzi IP letti correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          +CIFSR:APIP,"192.168.4.1"       \n
 *          +CIFSR:APMAC,"5e:cf:7f:8b:5c:d0"       \n
 *          +CIFSR:STAIP,"192.168.0.50"       \n
 *          +CIFSR:STAMAC,"5c:cf:7f:8b:5c:d0"
 */ 
unsigned char ESP8266_read_ip(unsigned int _timeout);  

/** 
 * Abilita o disabilita le connessioni multiple (massimo 4)
 * @param   _on_off: ON/OFF (unsigned char)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere impostato in modalit normale (CIPMODE = 0, default)
 * @note   
 *          - Tutte le funzioni sono implementate per funzionare in modalit connessioni multiple. impostare ESP8266_multiple_con(ON, 200)
 *          - Si pu impostare solo se non ci sono connessioni gi attive    
 *          - Prima di impostare disconnettere eventuali connessioni gi attive ed eliminare eventuali server TCP
 *          - Se esiste gi una connessione attiva torna ERROR      
 */ 
unsigned char ESP8266_multiple_con(unsigned char _on_off,  unsigned int _timeout);

/** 
 * Crea un server ed abilita monitor connessioni in ingresso
 * @param   _port[]: porta di connessione (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (server creato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 * @note   
 *          - Si pu impostare solo se non ci sono connessioni gi attive    
 *          - Prima di impostare disconnettere eventuali connessioni gi attive ed eliminare eventuali server TCP 
 */ 
unsigned char ESP8266_create_server(const unsigned char _port[],   unsigned int _timeout);

/** 
 * Imposta il tempo di timeout per la disconnessione di un client connesso al server ma senza comunicazione 
 * @param   _time[]: secondi senza comunicazione con il client da attendere prima di disconnettere il client 0~7200 secondi (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre     
 *          - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())        
 *          - il server deve essere attivo (ESP8266_create_server())
 * @note    
 *          - Impostare il timeout a 0 significa che il client non verr mai disconnesso ma questa impostazione  da evitare.
 *          - Torna al valore di default al riavvio (180)
 */ 
unsigned char ESP8266_set_server_timeout(const unsigned char _time[],  unsigned int _timeout);

/** 
 * Legge il tempo di timeout per la disconnessione di un client connesso al server ma senza comunicazione 
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (parametro letto correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          es: 3600 OK (60 minuti)
 */        
unsigned char ESP8266_read_server_timeout(unsigned int _timeout);

/** 
 * Chiude il server
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (server chiuso correttamente)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 */
unsigned char ESP8266_close_server(unsigned int _timeout);

/** 
 * Ottiene l'indirizzo IP del sito richiesto  
 * @param   _domain[]: indirizzo internet (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x01: ESP8266_RESP_OK (dominio trovato)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita) 
 * @pre 
 *          - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *          - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * @note    In esp8266_buffer torna i dati letti:       \n
 *          es: 80.88.85.16          
 */
unsigned char ESP8266_get_dns_ip(const unsigned char _domain[], unsigned int _timeout);

/** 
 * Si connette via TCP al sito richiesto
 * @param   _connection: numero della connessione da usare 0~4 (unsigned char)
 * @param   _domain[]: dominio a cui connettersi, indirizzo o IP (stringa)
 * @param   _port[]: porta di connessione (stringa)
 * @param   _time[]: tempo per mantenere la connessione attiva (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *          - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *          - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *          - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *          - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *          - 0x40: ESP8266_CON_0_CONNECT (connessione 0 connessa)
 *          - 0x41: ESP8266_CON_0_CONNECT (connessione 1 connessa)
 *          - 0x42: ESP8266_CON_0_CONNECT (connessione 2 connessa)
 *          - 0x43: ESP8266_CON_0_CONNECT (connessione 3 connessa)
 *          - 0x44: ESP8266_CON_0_CONNECT (connessione 4 connessa)
 *          - 0x22: ESP8266_RESP_ALREADY_CON (connessione gi esistente)
 *          - 0x23: ESP8266_RESP_DNS_FAIL (indirizzo non trovato)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @pre 
 *          - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *          - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *          - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * @note     Il comando pu essere usato anche in modalit connessione singola, ma non  implementato. Usare solo la modalit connessioni multiple
 */
unsigned char ESP8266_open_tcp_connection(unsigned char _connection, const unsigned char _domain[], const unsigned char _port[], const unsigned char _time[], unsigned int _timeout);

/** 
 * Si connette via UDP al sito richiesto
 * @param   _connection: numero della connessione da usare 0~4 (unsigned char)
 * @param   _domain[]: dominio a cui connettersi, indirizzo o IP (stringa)
 * @param   _port[]: porta di connessione (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR) 
 *          - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *          - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *          - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *          - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *          - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *          - 0x40: ESP8266_CON_0_CONNECT (connessione 0 connessa)
 *          - 0x41: ESP8266_CON_0_CONNECT (connessione 1 connessa)
 *          - 0x42: ESP8266_CON_0_CONNECT (connessione 2 connessa)
 *          - 0x43: ESP8266_CON_0_CONNECT (connessione 3 connessa)
 *          - 0x44: ESP8266_CON_0_CONNECT (connessione 4 connessa)
 *          - 0x22: ESP8266_RESP_ALREADY_CON (connessione gi esistente)
 *          - 0x23: ESP8266_RESP_DNS_FAIL (indirizzo non trovato)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @pre 
 *          - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *          - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *          - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * @note    Il comando pu essere usato anche in modalit connessione singola, ma non  implementato. Usare solo la modalit connessioni multiple
 */
unsigned char ESP8266_open_udp_connection(unsigned char _connection, const unsigned char _domain[], const unsigned char _port[], unsigned int _timeout);

/** 
 * Chiude la connessione TCP o UDP selezionata
 * @param   _connection: numero della connessione da chiudere 0~5 (5 = tutte le connessioni) (unsigned char)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *          - 0x24: ESP8266_RESP_UNLINK (connessione inesistente)
 *          - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *          - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *          - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *          - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *          - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *          - 0x50: ESP8266_CON_0_CLOSE (connessione 0 chiusa)
 *          - 0x51: ESP8266_CON_1_CLOSE (connessione 1 chiusa)
 *          - 0x52: ESP8266_CON_2_CLOSE (connessione 2 chiusa)
 *          - 0x53: ESP8266_CON_3_CLOSE (connessione 3 chiusa)
 *          - 0x54: ESP8266_CON_4_CLOSE (connessione 4 chiusa)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @pre 
 *          - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *          - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *          - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * @note    In disconnessione di tutte le connessioni torna in successione 0,CLOSE 2,CLOSE relativo alle connessioni disconnesse, in questo caso
 *          leggere la risposta direttamente in esp8266_buffer
 */
unsigned char ESP8266_close_connection(unsigned char _connection, unsigned int _timeout);

/** 
 * Invia una stringa di dati dalla lunghezza fissa alla connessione selezionata.
 * @param   _connection: numero della connessione alla quale inviare i dati 0~4 (unsigned char)
 * @param   _length[]: lunghezza della stringa da inviare (stringa)
 * @param   _string[]: stringa da inviare (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *          - 0x01: Dati inviati correttamente
 *          - 0x1F: ESP8266_RESP_SEND_FAIL (invio dati fallito)
 *          - 0x20: ESP8266_RESP_LINK_NOT_VALID (connessione non valida)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @pre     - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *          - La connessione deve essere attiva
 * @note    - Se si invia una stringa pi corta rispetto all lunghezza dichiarata il modulo resta bloccato finch non vengono inviati tutti i dati
 *          - Impostare un timeout relativamente alto per assicurarsi di ricevere una risposta
 */ 
unsigned char ESP8266_send_data_over_con_l(unsigned char _connection, const unsigned char _length[],const unsigned char _string[], unsigned int _timeout);

/** 
 * Invia una stringa di dati alla connessione selezionata. La trasmissione verr terminata a fine stringa o alla lunghezza impostata.     \n
 * @param   _connection: numero della connessione alla quale inviare i dati 0~4 (unsigned char)
 * @param   _length[]: lunghezza della stringa da inviare (stringa)
 * @param   _string[]: stringa da inviare (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *          - 0x01: Dati inviati correttamente
 *          - 0x1F: ESP8266_RESP_SEND_FAIL (invio dati fallito)
 *          - 0x20: ESP8266_RESP_LINK_NOT_VALID (connessione non valida)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @pre     
 *          - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *          - La connessione deve essere attiva
 * @note    - Assicurarsi che la lunghezza impostata copra la lunghezza massima della stringa da inviare.       \n
 *          - Impostare un timeout relativamente alto per assicurarsi di ricevere una risposta
 */ 
unsigned char ESP8266_send_data_over_con(unsigned char _connection, const unsigned char _length[], const unsigned char _string[], unsigned int _timeout);

/** 
 * Controlla lo stato della connessione
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *          - 0xC2: ESP8266_CON_AP_AND_IP (stazione connessa ad accesso point ed indirizzo IP ottenuto)
 *          - 0xC3: ESP8266_STA_TCP_UDP_TRA (connessione TCP o UDP attiva)
 *          - 0xC4: ESP8266_STA_TCP_UDP_TRA_DIS (connessione TCP o UDP disconnessa)
 *          - 0xC5: ESP8266_STA_NOT_CON_AP (stazione non connessa ad Access Point)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @note    In esp8266_buffer torna eventuali valori della connessione       \n
 *          es: <link ID>, <type>, <remote_IP>, <remote_port>, <local_port>, <tetype>   
 */ 
unsigned char ESP8266_check_connection_status(unsigned int _timeout);

/** 
 * Esegue un ping all'indirizzo richiesto
 * @param   _domain[]: dominio a cui connettersi, indirizzo o IP (stringa)
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *          - 0x01: Ping eseguito
 *          - 0x25: ESP8266_RESP_PING_TO (timeout ping)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @pre     Il modulo ESP8266 deve essere connesso ad un Acces Point
 * @note    In esp8266_buffer torna il tempo del ping in ms:       \n
 *          es:214  
 */ 
unsigned char ESP8266_ping(const unsigned char _domain[], unsigned int _timeout);

/** 
 * Verifica la presenza di nuovi dati in arrivo
 * @param   void
 * @return  
 *          - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *          - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *          - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *          - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *          - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *          - 0x40: ESP8266_CON_0_CONNECT (connessione 0 connessa)
 *          - 0x41: ESP8266_CON_0_CONNECT (connessione 1 connessa)
 *          - 0x42: ESP8266_CON_0_CONNECT (connessione 2 connessa)
 *          - 0x43: ESP8266_CON_0_CONNECT (connessione 3 connessa)
 *          - 0x44: ESP8266_CON_0_CONNECT (connessione 4 connessa)
 *          - 0x50: ESP8266_CON_0_CONNECT (connessione 0 chiusa)
 *          - 0x51: ESP8266_CON_0_CONNECT (connessione 1 chiusa)
 *          - 0x52: ESP8266_CON_0_CONNECT (connessione 2 chiusa)
 *          - 0x53: ESP8266_CON_0_CONNECT (connessione 3 chiusa)
 *          - 0x54: ESP8266_CON_0_CONNECT (connessione 4 chiusa)
 *          - 0x60: ESP8266_CON_0_SEND (dati ricevuti da connessione 0)
 *          - 0x61: ESP8266_CON_0_SEND (dati ricevuti da connessione 1)
 *          - 0x62: ESP8266_CON_0_SEND (dati ricevuti da connessione 2)
 *          - 0x63: ESP8266_CON_0_SEND (dati ricevuti da connessione 3)
 *          - 0x64: ESP8266_CON_0_SEND (dati ricevuti da connessione 4)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *          - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * @note    In caso di nuovi dati in arrivo diversi dallo stato della connessione (CONNECT/FAIL/CLOSE) i dati ricevuti sono in esp8266_buffer  \n
 *          Resettare il buffer tramite la funzione ESP8266_reset_buffer() dopo ogni stringa rcevuta ed interpretata
 */
unsigned char ESP8266_check_incoming_data(void);


/** 
 * Resetta il buffer di ricezione e attende il tempo di timeout o il flag di risposta ricevuta
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0xFB: ESP8266_RESP_RECEIVED (risposta ricevuta)
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 */ 
unsigned char ESP8266_wait_resp(unsigned int _timeout);

/** 
 * Decodifica la risposta ricevuta
 * @param   void
 * @return  
 *          tabella excel
 */ 
unsigned char ESP8266_decode_resp(void);

/** 
 * Attende il tempo di timeout o il flag di risposta ricevuta, a risposta ricevuta pulisce la stringa da /r/n e spazi e decodifica la risposta
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          Codifica ritorno risposte decodificate da ESP8266_decode_resp()
 */ 
unsigned char ESP8266_wait_decode_resp(unsigned int _timeout); 

/** 
 * Cerca la parola ready alla fine del buffer di ricezione
 * @param   _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * @return  
 *          - 0xFC: EPS8266_NOT_READY (modulo ESP8266 ha dato risposta ma non  stata trovata la parola ready
 *          - 0xFD: EPS8266_READY (modulo ESP8266 pronto)     
 *          - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 */ 
unsigned char ESP8266_check_ready(unsigned int _timeout);

/** 
 * Azzera il buffer di ricezione
 * @param   void
 * @return  void
 */ 
void ESP8266_reset_buffer(void);

/** 
 * Cancella parte del buffer di ricezione spostando all'inizio i byte rimanenti
 * @param   _n: numero di byte da cancellare all'inizio del buffer di ricezione (unsigned char)
 * @return  void
 */ 
void ESP8266_clean_buffer(unsigned char _n);

/** 
 * Cancella il buffer di ricezione fino al carattere indicato spostando all'inizio i byte rimanenti (carattere compreso)
 * @param   _char: ultimo carattere da cancellare (unsigned char)
 * @return  void
 */ 
void ESP8266_clean_buffer_until_char(unsigned char _char);

/** 
 * Imposta il timer dedicato alla ricezione dei dati
 * @param   void
 * @return  void
 */ 
void ESP8266_set_timer(void);