 /* ============================================================================================================================================= /
 * LIBRARY:         SPI
 * FILE:            SPI_init.h
 * LAST REV:        2.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/04/2014
 * UPDATED:         17/04/2014
 *
 *
 * DESCRIP:         Parametri e prototipi per libreria SPI_init.c
/* ============================================================================================================================================ */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#ifdef SPI_MODULE_1                                                                     // definizione linee a seconda del modulo SPI e del PIC usato
    #if defined (_18F4550) || defined (_18F2550)
        #define SPI_SCK_TRIS    TRISBbits.TRISB1
        #define SPI_SDI_TRIS    TRISBbits.TRISB0
        #define SPI_SDO_TRIS    TRISCbits.TRISC7
        #define SPI_SS_TRIS     TRISAbits.TRISA5
    #elif defined (_18F46K22) || defined (_18F26K22) || defined (_18LF26K22)
        #define SPI_SCK_TRIS    TRISCbits.TRISC3
        #define SPI_SDI_TRIS    TRISCbits.TRISC4
        #define SPI_SDO_TRIS    TRISCbits.TRISC5
        #define SPI_SS_TRIS     TRISAbits.TRISA5
    #elif defined _18F67K22
        #define SPI_SCK_TRIS    TRISCbits.TRISC3
        #define SPI_SDI_TRIS    TRISCbits.TRISC4
        #define SPI_SDO_TRIS    TRISCbits.TRISC5
        #define SPI_SS_TRIS     TRISFbits.TRISF7
    #elif defined _12F1840
        #define SPI_SCK_TRIS    TRISAbits.TRISA1
        #define SPI_SDI_TRIS    TRISAbits.TRISA2
        #if (SPI_SDO_RA == 0)
            #define SPI_SDO_TRIS    TRISAbits.TRISA0   
        #elif (SPI_SDO_RA == 4)
            #define SPI_SDO_TRIS    TRISAbits.TRISA4   
        #else
            #error SPI_SDO PIN non valido, impostare correttamente SPI_SDO_RA in SPI_init_settings.h
        #endif
    
        #if (SPI_SS_RA == 3)
            #define SPI_SS_TRIS    TRISAbits.TRISA2   
        #elif (SPI_SDO_RA == 0)
            #define SPI_SS_TRIS    TRISAbits.TRISA0   
        #else
            #error SPI_SS PIN non valido, impostare correttamente SPI_SS_RA in SPI_init_settings.h
        #endif
    #else
        #error PIC non Gestito. Definire costanti PIN di uscita per il nuovo PIC in SPI_init.h
    #endif
#endif

#ifdef SPI_MODULE_2
    #ifdef _18F46K22
        #define SPI2_SCK_TRIS    TRISDbits.TRISD0
        #define SPI2_SDI_TRIS    TRISDbits.TRISD1
        #define SPI2_SDO_TRIS    TRISDbits.TRISD4
        #define SPI2_SS_TRIS     TRISDbits.TRISD3
    #elif defined _18F26K22 || defined (_18LF26K22)
        #define SPI2_SCK_TRIS    TRISBbits.TRISB1
        #define SPI2_SDI_TRIS    TRISBbits.TRISB2
        #define SPI2_SDO_TRIS    TRISBbits.TRISB3
        #define SPI2_SS_TRIS     TRISBbits.TRISB0
    #elif defined _18F67K22
        #define SPI2_SCK_TRIS    TRISDbits.TRISD6
        #define SPI2_SDI_TRIS    TRISDbits.TRISD5
        #define SPI2_SDO_TRIS    TRISDbits.TRISD4
        #define SPI2_SS_TRIS     TRISDbits.TRISD7    
    #elif defined (_18F4550) || defined (_18F2550) ||  defined (_12F1840)
        #error il PIC selezionato non ha il secondo modulo SPI
    #else
        #error PIC non Gestito. Definire costanti PIN di uscita per il nuovo PIC in SPI_init.h
    #endif

//    #ifdef __18F67K22
//        #define SSP2IF  PIR2bits.SSP2IF
//    #else
//        #define SSP2IF  PIR3bits.SSP2IF
//    #endif
#endif

/* RINOMINAZIONE REGISTRI PIC12F1840 ========================================================================================================== */
#ifdef __12F1840
    #define SSPCON1         SSP1CON1
    #define SSPSTAT         SSP1STAT
    #define SSPIF           SSP1IF
    #define SSPBUF          SSP1BUF
#endif

/* RINOMINAZIONE REGISTRI PIC18F46K22 ========================================================================================================= */
//#ifdef _18F46K22
//    #define SSPCON1         SSP1CON1
//    #define SSPSTAT         SSP1STAT
//    #define SSPIF           SSP1IF
//    #define SSPBUF          SSP1BUF
//#endif

/* COSTANTI =================================================================================================================================== */
#define SAMPLE_AT_END       1                                                           // SMP = 1: Input data sampled at end of data output time
#define SAMPLE_AT_MIDDLE    0                                                           // SMP = 0: Input data sampled at middle of data output time
#define TRA_ACTIVE_TO_IDLE  1                                                           // CKE = 1: Transmit occurs on transition from active to Idle clock state
#define TRA_IDLE_TO_ACTIVE  0                                                           // CKE = 0: Transmit occurs on transition from Idle to active clock state
#define IDLE_IS_HIGH        1                                                           // CKP = 1: Idle state for clock is a high level
#define IDLE_IS_LOW         0                                                           // CKP = 0: Idle state for clock is a low level
#define MASTER_FOSC_4       0                                                           // SSPM = 0000: SPI Master mode, clock = FOSC/4(3
#define MASTER_FOSC_16      1                                                           // SSPM = 0001: SPI Master mode, clock = FOSC/16(3)
#define MASTER_FOSC_64      2                                                           // SSPM = 0010: SPI Master mode, clock = FOSC/64(3)
#define MASTER_TMR2_2       3                                                           // SSPM = 0011: SPI Master mode, clock = TMR2 output/2(3)
#define SLAVE_WITH_SS       4                                                           // SSPM = 0100: SPI Slave mode, clock = SCK pin, SS pin control enabled(3)
#define SLAVE_WO_SS         5                                                           // SSPM = 0101: SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin(3)

#define IN                  1
#define OUT                 0

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file SPI_init.c
 * Libreria per la gestione del bus SPI primario e secondario (se disponibile). Al momento sono state sviluppate solo le funzionalit  per dispositivo MASTER.
 * - Per un corretto funzionamento:
 *  + Impostare i moduli SPI abilitati e le linee utilizzate in SPI_init_settings.h
 *  + Impostare il modulo SPI specifico per il dispositivo nei settaggi dei vari dispositivi (se il dispositivo funziona senza libreria specifica basta abilitare il modulo in SPI_init_settings.h
 * - Files:
 *  + SPI_init.c: Codice funzioni
 *  + SPI_init.h: Parametri e prototipi
 *  + SPI_init_settings.h: Impostazioni specifiche per progetto
 */

/**
 * Abilita il protocollo SPI sul modulo primario con i parametri desiderati
 * @param   _smp:  
 *              - SAMPLE_AT_END    = Input data sampled at end of data output time  
 *              - SAMPLE_AT_MIDDLE = Input data sampled at middle of data output time  
 * @param   _cke:  
 *              - TRA_ACTIVE_TO_IDLE = Transmit occurs on transition from active to Idle clock state  
 *              - TRA_IDLE_TO_ACTIVE = Transmit occurs on transition from Idle to active clock state  
 * @param   _ckp:  \n
 *              - IDLE_IS_HIGH = Idle state for clock is a high level  
 *              - IDLE_IS_LOW = Idle state for clock is a low level  
 * @param   _sspm:  
 *              - MASTER_FOSC_4  = SPI Master mode, clock = FOSC/4
 *              - MASTER_FOSC_16 = SPI Master mode, clock = FOSC/16
 *              - MASTER_FOSC_64 = SPI Master mode, clock = FOSC/64
 *              - MASTER_TMR2_2  = SPI Master mode, clock = TMR2 output/2 (impostare timer 2 per ottenere la frequenza di clock desiderata)
 *              - SLAVE_WITH_SS  = SPI Slave mode, clock = SCK pin, SS pin control enabled
 *              - SLAVE_WO_SS    = SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin
 * @return  void
 */
void SPI_enable(unsigned char _smp, unsigned char _cke, unsigned char _ckp, unsigned char _sspm);

/**
 * Abilita il protocollo SPI sul modulo secondario con i parametri desiderati
 * @param   _smp:  
 *              - SAMPLE_AT_END    = Input data sampled at end of data output time  
 *              - SAMPLE_AT_MIDDLE = Input data sampled at middle of data output time  
 * @param   _cke:  
 *              - TRA_ACTIVE_TO_IDLE = Transmit occurs on transition from active to Idle clock state  
 *              - TRA_IDLE_TO_ACTIVE = Transmit occurs on transition from Idle to active clock state  
 * @param   _ckp:  
 *              - IDLE_IS_HIGH = Idle state for clock is a high level  
 *              - IDLE_IS_LOW = Idle state for clock is a low level  
 * @param   _sspm:  
 *              - MASTER_FOSC_4  = SPI Master mode, clock = FOSC/4  
 *              - MASTER_FOSC_16 = SPI Master mode, clock = FOSC/16  
 *              - MASTER_FOSC_64 = SPI Master mode, clock = FOSC/64( 
 *              - MASTER_TMR2_2  = SPI Master mode, clock = TMR2 output/2 (impostare timer 2 per ottenere la frequenza di clock desiderata)
 *              - SLAVE_WITH_SS  = SPI Slave mode, clock = SCK pin, SS pin control enabled
 *              - SLAVE_WO_SS    = SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin
 * @return  void
 */
void SPI2_enable(unsigned char _smp, unsigned char _cke, unsigned char _ckp, unsigned char _sspm);

/**
 * Disabilita il protocollo SPI del modulo primario
 * @param   void
 * @return  void
 */
void SPI_disable(void);

/**
 * Disabilita il protocollo SPI del modulo secondario
 * @param   void
 * @return  void
 */
void SPI2_disable(void);

/**
 * Invia il dato via SPI sul modulo primario
 * @param   _byte: dato da inviare (unsigned char)
 * @return  -1 in caso di BUS collision
 */
signed char SPI_send_byte(unsigned char _byte);

/**
 * Invia il dato via SPI sul modulo secondario
 * @param   _byte: dato da inviare (unsigned char)
 * @return  -1 in caso di BUS collision
 */
signed char SPI2_send_byte(unsigned char _byte);

/**
 * Riceve il dato via SPI sul modulo primario in polling sul bit BF
 * @param   void
 * @return  dato ricevuto (unsigned char)
 */
unsigned char SPI_receive_byte(void);

/**
 * Riceve il dato via SPI sul modulo secondario in polling sul bit BF
 * @param   void
 * @return  dato ricevuto (unsigned char)
 */
unsigned char SPI2_receive_byte(void);

/**
 * Invia e riceve 1 byte via SPI sul modulo primario
 * @param   _data: dato da inviare (unsigned char)
 * @return  dato ricevuto (unsigned char)
 */
unsigned char SPI_exchange_byte(unsigned char _data);

/**
 * Invia e riceve 1 byte via SPI sul modulo secondario
 * @param   _data: dato da inviare (unsigned char)
 * @return  dato ricevuto (unsigned char)
 */
unsigned char SPI2_exchange_byte(unsigned char _data);
